searchData={"items":[{"type":"module","title":"Ecto.Adapters.SQL","doc":"This application provides functionality for working with\nSQL databases in `Ecto`.","ref":"Ecto.Adapters.SQL.html"},{"type":"module","title":"Built-in adapters - Ecto.Adapters.SQL","doc":"By default, we support the following adapters:\n\n  * `Ecto.Adapters.Postgres` for Postgres\n  * `Ecto.Adapters.MyXQL` for MySQL\n  * `Ecto.Adapters.Tds` for SQLServer","ref":"Ecto.Adapters.SQL.html#module-built-in-adapters"},{"type":"module","title":"Additional functions - Ecto.Adapters.SQL","doc":"If your `Ecto.Repo` is backed by any of the SQL adapters above,\nthis module will inject additional functions into your repository:\n\n  * `disconnect_all(interval, options \\\\ [])` -\n     shortcut for `Ecto.Adapters.SQL.disconnect_all/3`\n\n  * `explain(type, query, options \\\\ [])` -\n     shortcut for `Ecto.Adapters.SQL.explain/4`\n\n  * `query(sql, params, options \\\\ [])` -\n     shortcut for `Ecto.Adapters.SQL.query/4`\n\n  * `query!(sql, params, options \\\\ [])` -\n     shortcut for `Ecto.Adapters.SQL.query!/4`\n\n  * `query_many(sql, params, options \\\\ [])` -\n     shortcut for `Ecto.Adapters.SQL.query_many/4`\n\n  * `query_many!(sql, params, options \\\\ [])` -\n     shortcut for `Ecto.Adapters.SQL.query_many!/4`\n\n  * `to_sql(type, query)` -\n     shortcut for `Ecto.Adapters.SQL.to_sql/3`\n\nGenerally speaking, you must invoke those functions directly from\nyour repository, for example: `MyApp.Repo.query(\"SELECT true\")`.\nYou can also invoke them directly from `Ecto.Adapters.SQL`, but\nkeep in mind that in such cases features such as \"dynamic repositories\"\nwon't be available.","ref":"Ecto.Adapters.SQL.html#module-additional-functions"},{"type":"module","title":"Migrations - Ecto.Adapters.SQL","doc":"`ecto_sql` supports database migrations. You can generate a migration\nwith:\n\n    $ mix ecto.gen.migration create_posts\n\nThis will create a new file inside `priv/repo/migrations` with the\n`change` function. Check `Ecto.Migration` for more information.\n\nTo interface with migrations, developers typically use mix tasks:\n\n  * `mix ecto.migrations` - lists all available migrations and their status\n  * `mix ecto.migrate` - runs a migration\n  * `mix ecto.rollback` - rolls back a previously run migration\n\nIf you want to run migrations programmatically, see `Ecto.Migrator`.","ref":"Ecto.Adapters.SQL.html#module-migrations"},{"type":"module","title":"SQL sandbox - Ecto.Adapters.SQL","doc":"`ecto_sql` provides a sandbox for testing. The sandbox wraps each\ntest in a transaction, making sure the tests are isolated and can\nrun concurrently. See `Ecto.Adapters.SQL.Sandbox` for more information.","ref":"Ecto.Adapters.SQL.html#module-sql-sandbox"},{"type":"module","title":"Structure load and dumping - Ecto.Adapters.SQL","doc":"If you have an existing database, you may want to dump its existing\nstructure and make it reproducible from within Ecto. This can be\nachieved with two Mix tasks:\n\n  * `mix ecto.load` - loads an existing structure into the database\n  * `mix ecto.dump` - dumps the existing database structure to the filesystem\n\nFor creating and dropping databases, see `mix ecto.create`\nand `mix ecto.drop` that are included as part of Ecto.","ref":"Ecto.Adapters.SQL.html#module-structure-load-and-dumping"},{"type":"module","title":"Custom adapters - Ecto.Adapters.SQL","doc":"Developers can implement their own SQL adapters by using\n`Ecto.Adapters.SQL` and by implementing the callbacks required\nby `Ecto.Adapters.SQL.Connection`  for handling connections and\nperforming queries. The connection handling and pooling for SQL\nadapters should be built using the `DBConnection` library.\n\nWhen using `Ecto.Adapters.SQL`, the following options are required:\n\n  * `:driver` (required) - the database driver library.\n    For example: `:postgrex`","ref":"Ecto.Adapters.SQL.html#module-custom-adapters"},{"type":"function","title":"Ecto.Adapters.SQL.disconnect_all/3","doc":"Forces all connections in the repo pool to disconnect within the given interval.\n\nOnce this function is called, the pool will disconnect all of its connections\nas they are checked in or as they are pinged. Checked in connections will be\nrandomly disconnected within the given time interval. Pinged connections are\nimmediately disconnected - as they are idle (according to `:idle_interval`).\n\nIf the connection has a backoff configured (which is the case by default),\ndisconnecting means an attempt at a new connection will be done immediately\nafter, without starting a new process for each connection. However, if backoff\nhas been disabled, the connection process will terminate. In such cases,\ndisconnecting all connections may cause the pool supervisor to restart\ndepending on the max_restarts/max_seconds configuration of the pool,\nso you will want to set those carefully.\n\nFor convenience, this function is also available in the repository:\n\n    iex> MyRepo.disconnect_all(60_000)\n    :ok","ref":"Ecto.Adapters.SQL.html#disconnect_all/3"},{"type":"function","title":"Ecto.Adapters.SQL.explain/4","doc":"Executes an EXPLAIN statement or similar for the given query according to its kind and the\nadapter in the given repository.","ref":"Ecto.Adapters.SQL.html#explain/4"},{"type":"function","title":"Examples - Ecto.Adapters.SQL.explain/4","doc":"# Postgres\n    iex> Ecto.Adapters.SQL.explain(Repo, :all, Post)\n    \"Seq Scan on posts p0  (cost=0.00..12.12 rows=1 width=443)\"\n\n    # MySQL\n    iex> Ecto.Adapters.SQL.explain(Repo, :all, from(p in Post, where: p.title == \"title\")) |> IO.puts()\n    +----+-------------+-------+------------+------+---------------+------+---------+------+------+----------+-------------+\n    | id | select_type | table | partitions | type | possible_keys | key  | key_len | ref  | rows | filtered | Extra       |\n    +----+-------------+-------+------------+------+---------------+------+---------+------+------+----------+-------------+\n    |  1 | SIMPLE      | p0    | NULL       | ALL  | NULL          | NULL | NULL    | NULL |    1 |    100.0 | Using where |\n    +----+-------------+-------+------------+------+---------------+------+---------+------+------+----------+-------------+\n\n    # Shared opts\n    iex> Ecto.Adapters.SQL.explain(Repo, :all, Post, analyze: true, timeout: 20_000)\n    \"Seq Scan on posts p0  (cost=0.00..11.70 rows=170 width=443) (actual time=0.013..0.013 rows=0 loops=1)\\nPlanning Time: 0.031 ms\\nExecution Time: 0.021 ms\"\n\nIt's safe to execute it for updates and deletes, no data change will be committed:\n\n    iex> Ecto.Adapters.SQL.explain(Repo, :update_all, from(p in Post, update: [set: [title: \"new title\"]]))\n    \"Update on posts p0  (cost=0.00..11.70 rows=170 width=449)\\n  ->  Seq Scan on posts p0  (cost=0.00..11.70 rows=170 width=449)\"\n\nThis function is also available under the repository with name `explain`:\n\n    iex> Repo.explain(:all, from(p in Post, where: p.title == \"title\"))\n    \"Seq Scan on posts p0  (cost=0.00..12.12 rows=1 width=443)\\n  Filter: ((title)::text = 'title'::text)\"","ref":"Ecto.Adapters.SQL.html#explain/4-examples"},{"type":"function","title":"Options - Ecto.Adapters.SQL.explain/4","doc":"Built-in adapters support passing `opts` to the EXPLAIN statement according to the following:\n\nAdapter          | Supported opts\n---------------- | --------------\nPostgrex         | `analyze`, `verbose`, `costs`, `settings`, `buffers`, `timing`, `summary`, `format`, `plan`\nMyXQL            | `format`\n\nAll options except `format` are boolean valued and default to `false`.\n\nThe allowed `format` values are `:map`, `:yaml`, and `:text`:\n  * `:map` is the deserialized JSON encoding.\n  * `:yaml` and `:text` return the result as a string.\n\nThe built-in adapters support the following formats:\n  * Postgrex: `:map`, `:yaml` and `:text`\n  * MyXQL: `:map` and `:text`\n\nThe `:plan` option in Postgrex can take the values `:custom` or `:fallback_generic`. When `:custom`\nis specified, the explain plan generated will consider the specific values of the query parameters\nthat are supplied. When using `:fallback_generic`, the specific values of the query parameters will\nbe ignored. `:fallback_generic` does not use PostgreSQL's built-in support for a generic explain\nplan (available as of PostgreSQL 16), but instead uses a special implementation that works for PostgreSQL\nversions 12 and above. Defaults to `:custom`.\n\nAny other value passed to `opts` will be forwarded to the underlying adapter query function, including\nshared Repo options such as `:timeout`. Non built-in adapters may have specific behaviour and you should\nconsult their documentation for more details.\n\nFor version compatibility, please check your database's documentation:\n\n  * _Postgrex_: [PostgreSQL doc](https://www.postgresql.org/docs/current/sql-explain.html).\n  * _MyXQL_: [MySQL doc](https://dev.mysql.com/doc/refman/8.0/en/explain.html).","ref":"Ecto.Adapters.SQL.html#explain/4-options"},{"type":"function","title":"Ecto.Adapters.SQL.query/4","doc":"Runs a custom SQL query on the given repo.\n\nIn case of success, it must return an `:ok` tuple containing\na map with at least two keys:\n\n  * `:num_rows` - the number of rows affected\n\n  * `:rows` - the result set as a list. `nil` may be returned\n    instead of the list if the command does not yield any row\n    as result (but still yields the number of affected rows,\n    like a `delete` command without returning would)","ref":"Ecto.Adapters.SQL.html#query/4"},{"type":"function","title":"Options - Ecto.Adapters.SQL.query/4","doc":"* `:log` - When false, does not log the query\n  * `:timeout` - Execute request timeout, accepts: `:infinity` (default: `15000`);","ref":"Ecto.Adapters.SQL.html#query/4-options"},{"type":"function","title":"Examples - Ecto.Adapters.SQL.query/4","doc":"iex> Ecto.Adapters.SQL.query(MyRepo, \"SELECT $1::integer + $2\", [40, 2])\n    {:ok, %{rows: [[42]], num_rows: 1}}\n\nFor convenience, this function is also available under the repository:\n\n    iex> MyRepo.query(\"SELECT $1::integer + $2\", [40, 2])\n    {:ok, %{rows: [[42]], num_rows: 1}}","ref":"Ecto.Adapters.SQL.html#query/4-examples"},{"type":"function","title":"Ecto.Adapters.SQL.query!/4","doc":"Same as `query/4` but raises on invalid queries.","ref":"Ecto.Adapters.SQL.html#query!/4"},{"type":"function","title":"Ecto.Adapters.SQL.query_many/4","doc":"Runs a custom SQL query that returns multiple results on the given repo.\n\nIn case of success, it must return an `:ok` tuple containing\na list of maps with at least two keys:\n\n  * `:num_rows` - the number of rows affected\n\n  * `:rows` - the result set as a list. `nil` may be returned\n    instead of the list if the command does not yield any row\n    as result (but still yields the number of affected rows,\n    like a `delete` command without returning would)","ref":"Ecto.Adapters.SQL.html#query_many/4"},{"type":"function","title":"Options - Ecto.Adapters.SQL.query_many/4","doc":"* `:log` - When false, does not log the query\n  * `:timeout` - Execute request timeout, accepts: `:infinity` (default: `15000`);","ref":"Ecto.Adapters.SQL.html#query_many/4-options"},{"type":"function","title":"Examples - Ecto.Adapters.SQL.query_many/4","doc":"iex> Ecto.Adapters.SQL.query_many(MyRepo, \"SELECT $1; SELECT $2;\", [40, 2])\n    {:ok, [%{rows: [[40]], num_rows: 1}, %{rows: [[2]], num_rows: 1}]}\n\nFor convenience, this function is also available under the repository:\n\n    iex> MyRepo.query_many(\"SELECT $1; SELECT $2;\", [40, 2])\n    {:ok, [%{rows: [[40]], num_rows: 1}, %{rows: [[2]], num_rows: 1}]}","ref":"Ecto.Adapters.SQL.html#query_many/4-examples"},{"type":"function","title":"Ecto.Adapters.SQL.query_many!/4","doc":"Same as `query_many/4` but raises on invalid queries.","ref":"Ecto.Adapters.SQL.html#query_many!/4"},{"type":"function","title":"Ecto.Adapters.SQL.stream/4","doc":"Returns a stream that runs a custom SQL query on given repo when reduced.\n\nIn case of success it is a enumerable containing maps with at least two keys:\n\n  * `:num_rows` - the number of rows affected\n\n  * `:rows` - the result set as a list. `nil` may be returned\n    instead of the list if the command does not yield any row\n    as result (but still yields the number of affected rows,\n    like a `delete` command without returning would)\n\nIn case of failure it raises an exception.\n\nIf the adapter supports a collectable stream, the stream may also be used as\nthe collectable in `Enum.into/3`. Behaviour depends on the adapter.","ref":"Ecto.Adapters.SQL.html#stream/4"},{"type":"function","title":"Options - Ecto.Adapters.SQL.stream/4","doc":"* `:log` - When false, does not log the query\n  * `:max_rows` - The number of rows to load from the database as we stream","ref":"Ecto.Adapters.SQL.html#stream/4-options"},{"type":"function","title":"Examples - Ecto.Adapters.SQL.stream/4","doc":"iex> Ecto.Adapters.SQL.stream(MyRepo, \"SELECT $1::integer + $2\", [40, 2]) |> Enum.to_list()\n    [%{rows: [[42]], num_rows: 1}]","ref":"Ecto.Adapters.SQL.html#stream/4-examples"},{"type":"function","title":"Ecto.Adapters.SQL.table_exists?/3","doc":"Check if the given `table` exists.\n\nReturns `true` if the `table` exists in the `repo`, otherwise `false`.\nThe table is checked against the current database/schema in the connection.","ref":"Ecto.Adapters.SQL.html#table_exists?/3"},{"type":"function","title":"Ecto.Adapters.SQL.to_sql/3","doc":"Converts the given query to SQL according to its kind and the\nadapter in the given repository.","ref":"Ecto.Adapters.SQL.html#to_sql/3"},{"type":"function","title":"Examples - Ecto.Adapters.SQL.to_sql/3","doc":"The examples below are meant for reference. Each adapter will\nreturn a different result:\n\n    iex> Ecto.Adapters.SQL.to_sql(:all, Repo, Post)\n    {\"SELECT p.id, p.title, p.inserted_at, p.created_at FROM posts as p\", []}\n\n    iex> Ecto.Adapters.SQL.to_sql(:update_all, Repo,\n                                  from(p in Post, update: [set: [title: ^\"hello\"]]))\n    {\"UPDATE posts AS p SET title = $1\", [\"hello\"]}\n\nThis function is also available under the repository with name `to_sql`:\n\n    iex> Repo.to_sql(:all, Post)\n    {\"SELECT p.id, p.title, p.inserted_at, p.created_at FROM posts as p\", []}","ref":"Ecto.Adapters.SQL.html#to_sql/3-examples"},{"type":"type","title":"Ecto.Adapters.SQL.query_params/0","doc":"","ref":"Ecto.Adapters.SQL.html#t:query_params/0"},{"type":"type","title":"Ecto.Adapters.SQL.query_result/0","doc":"","ref":"Ecto.Adapters.SQL.html#t:query_result/0"},{"type":"module","title":"Ecto.Adapters.SQL.Sandbox","doc":"A pool for concurrent transactional tests.\n\nThe sandbox pool is implemented on top of an ownership mechanism.\nWhen started, the pool is in automatic mode, which means the\nrepository will automatically check connections out as with any\nother pool.\n\nThe `mode/2` function can be used to change the pool mode from\nautomatic to either manual or shared. In the latter two modes,\nthe connection must be explicitly checked out before use.\nWhen explicit checkouts are made, the sandbox will wrap the\nconnection in a transaction by default and control who has\naccess to it. This means developers have a safe mechanism for\nrunning concurrent tests against the database.","ref":"Ecto.Adapters.SQL.Sandbox.html"},{"type":"module","title":"Database support - Ecto.Adapters.SQL.Sandbox","doc":"While both PostgreSQL and MySQL support SQL Sandbox, only PostgreSQL\nsupports concurrent tests while running the SQL Sandbox. Therefore, do\nnot run concurrent tests with MySQL as you may run into deadlocks due to\nits transaction implementation.","ref":"Ecto.Adapters.SQL.Sandbox.html#module-database-support"},{"type":"module","title":"Example - Ecto.Adapters.SQL.Sandbox","doc":"The first step is to configure your database to use the\n`Ecto.Adapters.SQL.Sandbox` pool. You set those options in your\n`config/config.exs` (or preferably `config/test.exs`) if you\nhaven't yet:\n\n    config :my_app, Repo,\n      pool: Ecto.Adapters.SQL.Sandbox\n\nNow with the test database properly configured, you can write\ntransactional tests:\n\n    # At the end of your test_helper.exs\n    # Set the pool mode to manual for explicit checkouts\n    Ecto.Adapters.SQL.Sandbox.mode(Repo, :manual)\n\n    defmodule PostTest do\n      # Once the mode is manual, tests can also be async\n      use ExUnit.Case, async: true\n\n      setup do\n        # Explicitly get a connection before each test\n        :ok = Ecto.Adapters.SQL.Sandbox.checkout(Repo)\n      end\n\n      setup tags do\n        pid = Ecto.Adapters.SQL.Sandbox.start_owner!(Repo, shared: not tags[:async])\n        on_exit(fn -> Ecto.Adapters.SQL.Sandbox.stop_owner(pid) end)\n        :ok\n      end\n\n      test \"create post\" do\n        # Use the repository as usual\n        assert %Post{} = Repo.insert!(%Post{})\n      end\n    end","ref":"Ecto.Adapters.SQL.Sandbox.html#module-example"},{"type":"module","title":"Collaborating processes - Ecto.Adapters.SQL.Sandbox","doc":"The example above is straight-forward because we have only\na single process using the database connection. However,\nsometimes a test may need to interact with multiple processes,\nall using the same connection so they all belong to the same\ntransaction.\n\nBefore we discuss solutions, let's see what happens if we try\nto use a connection from a new process without explicitly\nchecking it out first:\n\n    setup do\n      # Explicitly get a connection before each test\n      :ok = Ecto.Adapters.SQL.Sandbox.checkout(Repo)\n    end\n\n    test \"calls worker that runs a query\" do\n      GenServer.call(MyApp.Worker, :run_query)\n    end\n\nThe test above will fail with an error similar to:\n\n    ** (DBConnection.OwnershipError) cannot find ownership process for #PID<0.35.0>\n\nThat's because the `setup` block is checking out the connection only\nfor the test process. Once the worker attempts to perform a query,\nthere is no connection assigned to it and it will fail.\n\nThe sandbox module provides two ways of doing so, via allowances or\nby running in shared mode.","ref":"Ecto.Adapters.SQL.Sandbox.html#module-collaborating-processes"},{"type":"module","title":"Allowances - Ecto.Adapters.SQL.Sandbox","doc":"The idea behind allowances is that you can explicitly tell a process\nwhich checked out connection it should use, allowing multiple processes\nto collaborate over the same connection. Let's give it a try:\n\n    test \"calls worker that runs a query\" do\n      allow = Process.whereis(MyApp.Worker)\n      Ecto.Adapters.SQL.Sandbox.allow(Repo, self(), allow)\n      GenServer.call(MyApp.Worker, :run_query)\n    end\n\nAnd that's it, by calling `allow/3`, we are explicitly assigning\nthe parent's connection (i.e. the test process' connection) to\nthe task.\n\nBecause allowances use an explicit mechanism, their advantage\nis that you can still run your tests in async mode. The downside\nis that you need to explicitly control and allow every single\nprocess. This is not always possible. In such cases, you will\nwant to use shared mode.","ref":"Ecto.Adapters.SQL.Sandbox.html#module-allowances"},{"type":"module","title":"Shared mode - Ecto.Adapters.SQL.Sandbox","doc":"Shared mode allows a process to share its connection with any other\nprocess automatically, without relying on explicit allowances.\nLet's change the example above to use shared mode:\n\n    setup do\n      # Explicitly get a connection before each test\n      :ok = Ecto.Adapters.SQL.Sandbox.checkout(Repo)\n      # Setting the shared mode must be done only after checkout\n      Ecto.Adapters.SQL.Sandbox.mode(Repo, {:shared, self()})\n    end\n\n    test \"calls worker that runs a query\" do\n      GenServer.call(MyApp.Worker, :run_query)\n    end\n\nBy calling `mode({:shared, self()})`, any process that needs\nto talk to the database will now use the same connection as the\none checked out by the test process during the `setup` block.\n\nMake sure to always check a connection out before setting the mode\nto `{:shared, self()}`.\n\nThe advantage of shared mode is that by calling a single function,\nyou will ensure all upcoming processes and operations will use that\nshared connection, without a need to explicitly allow them. The\ndownside is that tests can no longer run concurrently in shared mode.\n\nAlso, beware that if the test process terminates while the worker is\nusing the connection, the connection will be taken away from the worker,\nwhich will error. Therefore it is important to guarantee the work is done\nbefore the test concludes. In the example above, we are using a `call`,\nwhich is synchronous, avoiding the problem, but you may need to explicitly\nflush the worker or terminate it under such scenarios in your tests.","ref":"Ecto.Adapters.SQL.Sandbox.html#module-shared-mode"},{"type":"module","title":"Summing up - Ecto.Adapters.SQL.Sandbox","doc":"There are two mechanisms for explicit ownerships:\n\n  * Using allowances - requires explicit allowances via `allow/3`.\n    Tests may run concurrently.\n\n  * Using shared mode - does not require explicit allowances.\n    Tests cannot run concurrently.","ref":"Ecto.Adapters.SQL.Sandbox.html#module-summing-up"},{"type":"module","title":"FAQ - Ecto.Adapters.SQL.Sandbox","doc":"When running the sandbox mode concurrently, developers may run into\nissues we explore in the upcoming sections.\n\n### \"owner exited\"\n\nIn some situations, you may see error reports similar to the one below:\n\n    23:59:59.999 [error] Postgrex.Protocol (#PID<>) disconnected:\n        ** (DBConnection.Error) owner #PID<> exited\n    Client #PID<> is still using a connection from owner\n\nSuch errors are usually followed by another error report from another\nprocess that failed while executing a database query.\n\nTo understand the failure, we need to answer the question: who are the\nowner and client processes? The owner process is the one that checks\nout the connection, which, in the majority of cases, is the test process,\nthe one running your tests. In other words, the error happens because\nthe test process has finished, either because the test succeeded or\nbecause it failed, while the client process was trying to get information\nfrom the database. Since the owner process, the one that owns the\nconnection, no longer exists, Ecto will check the connection back in\nand notify the client process using the connection that the connection\nowner is no longer available.\n\nThis can happen in different situations. For example, imagine you query\na GenServer in your test that is using a database connection:\n\n    test \"gets results from GenServer\" do\n      {:ok, pid} = MyAppServer.start_link()\n      Ecto.Adapters.SQL.Sandbox.allow(Repo, self(), pid)\n      assert MyAppServer.get_my_data_fast(timeout: 1000) == [...]\n    end\n\nIn the test above, we spawn the server and allow it to perform database\nqueries using the connection owned by the test process. Since we gave\na timeout of 1 second, in case the database takes longer than one second\nto reply, the test process will fail, due to the timeout, making the\n\"owner down\" message to be printed because the server process is still\nwaiting on a connection reply.\n\nIn some situations, such failures may be intermittent. Imagine that you\nallow a process that queries the database every half second:\n\n    test \"queries periodically\" do\n      {:ok, pid} = PeriodicServer.start_link()\n      Ecto.Adapters.SQL.Sandbox.allow(Repo, self(), pid)\n      # assertions\n    end\n\nBecause the server is querying the database from time to time, there is\na chance that, when the test exits, the periodic process may be querying\nthe database, regardless of test success or failure.\n\nTo address this, you can tell ExUnit to manage your processes:\n\n    test \"queries periodically\" do\n      pid = start_supervised!(PeriodicServer)\n      Ecto.Adapters.SQL.Sandbox.allow(Repo, self(), pid)\n      # assertions\n    end\n\nBy using `start_supervised!/1`, ExUnit guarantees the process finishes\nbefore your test (the connection owner).\n\nIn some situations, however, the dynamic processes are directly started\ninside a `DynamicSupervisor` or a `Task.Supervisor`. You can guarantee\nproper termination in such scenarios by adding an `on_exit` callback\nthat waits until all supervised children terminate:\n\n    on_exit(fn ->\n      for {_, pid, _, _} <- DynamicSupervisor.which_children(MyApp.DynamicSupervisor) do\n        ref = Process.monitor(pid)\n        assert_receive {:DOWN, ^ref, _, _, _}, :infinity\n      end\n    end)\n\n### \"owner timed out because it owned the connection for longer than Nms\"\n\nIn some situations, you may see error reports similar to the one below:\n\n    09:56:43.081 [error] Postgrex.Protocol (#PID<>) disconnected:\n        ** (DBConnection.ConnectionError) owner #PID<> timed out\n        because it owned the connection for longer than 120000ms\n\nIf you have a long running test (or you're debugging with IEx.pry),\nthe timeout for the connection ownership may be too short.  You can\nincrease the timeout by setting the `:ownership_timeout` options for\nyour repo config in `config/config.exs` (or preferably in `config/test.exs`):\n\n    config :my_app, MyApp.Repo,\n      ownership_timeout: NEW_TIMEOUT_IN_MILLISECONDS\n\nThe `:ownership_timeout` option is part of `DBConnection.Ownership`\nand defaults to 120000ms. Timeouts are given as integers in milliseconds.\n\nAlternately, if this is an issue for only a handful of long-running tests,\nyou can pass an `:ownership_timeout` option when calling\n`Ecto.Adapters.SQL.Sandbox.checkout/2` instead of setting a longer timeout\nglobally in your config.","ref":"Ecto.Adapters.SQL.Sandbox.html#module-faq"},{"type":"module","title":"Deferred constraints - Ecto.Adapters.SQL.Sandbox","doc":"Some databases allow to defer constraint validation to the transaction\ncommit time, instead of the particular statement execution time. This\nfeature, for instance, allows for a cyclic foreign key referencing.\nSince the SQL Sandbox mode rolls back transactions, tests might report\nfalse positives because deferred constraints are never checked by the\ndatabase. To manually force deferred constraints validation when using\nPostgreSQL use the following line right at the end of your test case:\n\n    Repo.query!(\"SET CONSTRAINTS ALL IMMEDIATE\")","ref":"Ecto.Adapters.SQL.Sandbox.html#module-deferred-constraints"},{"type":"module","title":"Database locks and deadlocks - Ecto.Adapters.SQL.Sandbox","doc":"Since the sandbox relies on concurrent transactional tests, there is\na chance your tests may trigger deadlocks in your database. This is\nspecially true with MySQL, where the solutions presented here are not\nenough to avoid deadlocks and therefore making the use of concurrent tests\nwith MySQL prohibited.\n\nHowever, even on databases like PostgreSQL, performance degradations or\ndeadlocks may still occur. For example, imagine a \"users\" table with a\nunique index on the \"email\" column. Now consider multiple tests are\ntrying to insert the same user email to the database. They will attempt\nto retrieve the same database lock, causing only one test to succeed and\nrun while all other tests wait for the lock.\n\nIn other situations, two different tests may proceed in a way that\neach test retrieves locks desired by the other, leading to a situation\nthat cannot be resolved, a deadlock. For instance:\n\n```text\nTransaction 1:                Transaction 2:\nbegin\n                              begin\nupdate posts where id = 1\n                              update posts where id = 2\n                              update posts where id = 1\nupdate posts where id = 2\n                      **deadlock**\n```\n\nThere are different ways to avoid such problems. One of them is\nto make sure your tests work on distinct data. Regardless of\nyour choice between using fixtures or factories for test data,\nmake sure you get a new set of data per test. This is specially\nimportant for data that is meant to be unique like user emails.\n\nFor example, instead of:\n\n    def insert_user do\n      Repo.insert!(%User{email: \"sample@example.com\"})\n    end\n\nprefer:\n\n    def insert_user do\n      Repo.insert!(%User{email: \"sample-#{counter()}@example.com\"})\n    end\n\n    defp counter do\n      System.unique_integer([:positive])\n    end\n\nIn fact, avoiding unique emails like above can also have a positive\nimpact on the test suite performance, as it reduces contention and\nwait between concurrent tests. We have heard reports where using\ndynamic values for uniquely indexed columns, as we did for email\nabove, made a test suite run between 2x to 3x faster.\n\nDeadlocks may happen in other circumstances. If you believe you\nare hitting a scenario that has not been described here, please\nreport an issue so we can improve our examples. As a last resort,\nyou can always disable the test triggering the deadlock from\nrunning asynchronously by setting  \"async: false\".","ref":"Ecto.Adapters.SQL.Sandbox.html#module-database-locks-and-deadlocks"},{"type":"function","title":"Ecto.Adapters.SQL.Sandbox.allow/4","doc":"Allows the `allow` process to use the same connection as `parent`.\n\n`allow` may be a PID or a locally registered name.\n\nIf the allowance is successful, this function returns `:ok`. If `allow` is already an\nowner or already allowed, it returns `{:already, :owner | :allowed}`. If `parent` has not\nchecked out a connection from the repo, it returns `:not_found`.","ref":"Ecto.Adapters.SQL.Sandbox.html#allow/4"},{"type":"function","title":"Ecto.Adapters.SQL.Sandbox.checkin/2","doc":"Checks in the connection back into the sandbox pool.","ref":"Ecto.Adapters.SQL.Sandbox.html#checkin/2"},{"type":"function","title":"Ecto.Adapters.SQL.Sandbox.checkout/2","doc":"Checks a connection out for the given `repo`.\n\nThe process calling `checkout/2` will own the connection\nuntil it calls `checkin/2` or until it crashes in which case\nthe connection will be automatically reclaimed by the pool.\n\nIf successful, returns `:ok`. If the caller already has a\nconnection, it returns `{:already, :owner | :allowed}`.","ref":"Ecto.Adapters.SQL.Sandbox.html#checkout/2"},{"type":"function","title":"Options - Ecto.Adapters.SQL.Sandbox.checkout/2","doc":"* `:sandbox` - when true the connection is wrapped in\n    a transaction. Defaults to true.\n\n  * `:isolation` - set the query to the given isolation level.\n\n  * `:ownership_timeout` - limits how long the connection can be\n    owned. Defaults to the value in your repo config in\n    `config/config.exs` (or preferably in `config/test.exs`), or\n    120000 ms if not set. The timeout exists for sanity checking\n    purposes, to ensure there is no connection leakage, and can\n    be bumped whenever necessary.","ref":"Ecto.Adapters.SQL.Sandbox.html#checkout/2-options"},{"type":"function","title":"Ecto.Adapters.SQL.Sandbox.mode/2","doc":"Sets the mode for the `repo` pool.\n\nThe modes can be:\n\n  * `:auto` - this is the default mode. When trying to use the repository,\n    processes can automatically checkout a connection without calling\n    `checkout/2` or `start_owner/2` before. This is the mode you will run\n    on before your test suite starts\n\n  * `:manual` - in this mode, the connection always has to be explicitly\n    checked before used. Other processes are allowed to use the same\n    connection if they are explicitly allowed via `allow/4`. You usually\n    set the mode to manual at the end of your `test/test_helper.exs` file.\n    This is also the mode you will run your async tests in\n\n  * `{:shared, pid}` - after checking out a connection in manual mode,\n    you can change the mode to `{:shared, pid}`, where pid is the process\n    that owns the connection, most often `{:shared, self()}`. This makes it\n    so all processes can use the same connection as the one owned by the\n    current process. This is the mode you will run your sync tests in\n\nWhenever you change the mode to `:manual` or `:auto`, all existing\nconnections are checked in. Therefore, it is recommend to set those\nmodes before your test suite starts, as otherwise you will check in\nconnections being used in any other test running concurrently.\n\nIf successful, returns `:ok` (this is always successful for `:auto`\nand `:manual` modes). It may return `:not_owner` or `:not_found`\nwhen setting `{:shared, pid}` and the given `pid` does not own any\nconnection for the repo. May return `:already_shared` if another\nprocess set the ownership mode to `{:shared, _}` and is still alive.","ref":"Ecto.Adapters.SQL.Sandbox.html#mode/2"},{"type":"function","title":"Ecto.Adapters.SQL.Sandbox.start_owner!/2","doc":"Starts a process that owns the connection and returns its pid.\n\nThe owner process is not linked to the caller, it is your responsibility to\nensure it will be stopped. In tests, this is done by terminating the pool\nin an `ExUnit.Callbacks.on_exit/2` callback:\n\n    setup tags do\n      pid = Ecto.Adapters.SQL.Sandbox.start_owner!(MyApp.Repo, shared: not tags[:async])\n      on_exit(fn -> Ecto.Adapters.SQL.Sandbox.stop_owner(pid) end)\n      :ok\n    end","ref":"Ecto.Adapters.SQL.Sandbox.html#start_owner!/2"},{"type":"function","title":"Options - Ecto.Adapters.SQL.Sandbox.start_owner!/2","doc":"* `:shared` - if `true`, the pool runs in the shared mode. Defaults to `false`\n\nThe remaining options are passed to `checkout/2`.","ref":"Ecto.Adapters.SQL.Sandbox.html#start_owner!/2-options"},{"type":"function","title":"Ecto.Adapters.SQL.Sandbox.stop_owner/1","doc":"Stops an owner process started by `start_owner!/2`.","ref":"Ecto.Adapters.SQL.Sandbox.html#stop_owner/1"},{"type":"function","title":"Ecto.Adapters.SQL.Sandbox.unboxed_run/2","doc":"Runs a function outside of the sandbox.","ref":"Ecto.Adapters.SQL.Sandbox.html#unboxed_run/2"},{"type":"behaviour","title":"Ecto.Migration","doc":"Migrations are used to modify your database schema over time.\n\nThis module provides many helpers for migrating the database,\nallowing developers to use Elixir to alter their storage in\na way that is database independent.\n\nMigrations typically provide two operations: `up` and `down`,\nallowing us to migrate the database forward or roll it back\nin case of errors.\n\nIn order to manage migrations, Ecto creates a table called\n`schema_migrations` in the database, which stores all migrations\nthat have already been executed. You can configure the name of\nthis table with the `:migration_source` configuration option\nand the name of the repository that manages it with `:migration_repo`.\n\nEcto locks the `schema_migrations` table when running\nmigrations, guaranteeing two different servers cannot run the same\nmigration at the same time.","ref":"Ecto.Migration.html"},{"type":"behaviour","title":"Creating your first migration - Ecto.Migration","doc":"Migrations are defined inside the \"priv/REPO/migrations\" where REPO\nis the last part of the repository name in underscore. For example,\nmigrations for `MyApp.Repo` would be found in \"priv/repo/migrations\".\nFor `MyApp.CustomRepo`, it would be found in \"priv/custom_repo/migrations\".\n\nEach file in the migrations directory has the following structure:\n\n```text\nNUMBER_NAME.exs\n```\n\nThe NUMBER is a unique number that identifies the migration. It is\nusually the timestamp of when the migration was created. The NAME\nmust also be unique and it quickly identifies what the migration\ndoes. For example, if you need to track the \"weather\" in your system,\nyou can start a new file at \"priv/repo/migrations/20190417140000_add_weather_table.exs\"\nthat will have the following contents:\n\n    defmodule MyRepo.Migrations.AddWeatherTable do\n      use Ecto.Migration\n\n      def up do\n        create table(\"weather\") do\n          add :city,    :string, size: 40\n          add :temp_lo, :integer\n          add :temp_hi, :integer\n          add :prcp,    :float\n\n          timestamps()\n        end\n      end\n\n      def down do\n        drop table(\"weather\")\n      end\n    end\n\nThe `up/0` function is responsible to migrate your database forward.\nthe `down/0` function is executed whenever you want to rollback.\nThe `down/0` function must always do the opposite of `up/0`.\nInside those functions, we invoke the API defined in this module,\nyou will find conveniences for managing tables, indexes, columns,\nreferences, as well as running custom SQL commands.\n\nTo run a migration, we generally use Mix tasks. For example, you can\nrun the migration above by going to the root of your project and\ntyping:\n\n    $ mix ecto.migrate\n\nYou can also roll it back by calling:\n\n    $ mix ecto.rollback --step 1\n\nNote rollback requires us to say how much we want to rollback.\nOn the other hand, `mix ecto.migrate` will always run all pending\nmigrations.\n\nIn practice, we don't create migration files by hand either, we\ntypically use `mix ecto.gen.migration` to generate the file with\nthe proper timestamp and then we just fill in its contents:\n\n    $ mix ecto.gen.migration add_weather_table\n\nFor the rest of this document, we will cover the migration APIs\nprovided by Ecto. For a in-depth discussion of migrations and how\nto use them safely within your application and data, see the\n[Safe Ecto Migrations guide](https://fly.io/phoenix-files/safe-ecto-migrations/).","ref":"Ecto.Migration.html#module-creating-your-first-migration"},{"type":"behaviour","title":"Mix tasks - Ecto.Migration","doc":"As seen above, Ecto provides many Mix tasks to help developers work\nwith migrations. We summarize them below:\n\n  * `mix ecto.gen.migration` - generates a\n    migration that the user can fill in with particular commands\n  * `mix ecto.migrate` - migrates a repository\n  * `mix ecto.migrations` - shows all migrations and their status\n  * `mix ecto.rollback` - rolls back a particular migration\n\nRun `mix help COMMAND` for more information on a particular command.\nFor a lower level API for running migrations, see `Ecto.Migrator`.","ref":"Ecto.Migration.html#module-mix-tasks"},{"type":"behaviour","title":"Change - Ecto.Migration","doc":"Having to write both `up/0` and `down/0` functions for every\nmigration is tedious and error prone. For this reason, Ecto allows\nyou to define a `change/0` callback with all of the code you want\nto execute when migrating and Ecto will automatically figure out\nthe `down/0` for you. For example, the migration above can be\nwritten as:\n\n    defmodule MyRepo.Migrations.AddWeatherTable do\n      use Ecto.Migration\n\n      def change do\n        create table(\"weather\") do\n          add :city,    :string, size: 40\n          add :temp_lo, :integer\n          add :temp_hi, :integer\n          add :prcp,    :float\n\n          timestamps()\n        end\n      end\n    end\n\nHowever, note that not all commands are reversible. Trying to rollback\na non-reversible command will raise an `Ecto.MigrationError`.\n\nA notable command in this regard is `execute/2`, which is reversible in\n`change/0` by accepting a pair of plain SQL strings. The first is run on\nforward migrations (`up/0`) and the second when rolling back (`down/0`).\n\nIf `up/0` and `down/0` are implemented in a migration, they take precedence,\nand `change/0` isn't invoked.","ref":"Ecto.Migration.html#module-change"},{"type":"behaviour","title":"Field Types - Ecto.Migration","doc":"The [Ecto primitive types](https://hexdocs.pm/ecto/Ecto.Schema.html#module-primitive-types) are mapped to the appropriate database\ntype by the various database adapters. For example, `:string` is\nconverted to `:varchar`, `:binary` to `:bytea` or `:blob`, and so on.\n\nIn particular, note that:\n\n  * the `:string` type in migrations by default has a limit of 255 characters.\n    If you need more or less characters, pass the `:size` option, such\n    as `add :field, :string, size: 10`. If you don't want to impose a limit,\n    most databases support a `:text` type or similar\n\n  * the `:binary` type in migrations by default has no size limit. If you want\n    to impose a limit, pass the `:size` option accordingly. In MySQL, passing\n    the size option changes the underlying field from \"blob\" to \"varbinary\"\n\nAny other type will be given as is to the database. For example, you\ncan use `:text`, `:char`, or `:varchar` as types. Types that have spaces\nin their names can be wrapped in double quotes, such as `:\"int unsigned\"`,\n`:\"time without time zone\"`, etc.","ref":"Ecto.Migration.html#module-field-types"},{"type":"behaviour","title":"Executing and flushing - Ecto.Migration","doc":"Most functions in this module, when executed inside of migrations, are not\nexecuted immediately. Instead they are performed after the relevant `up`,\n`change`, or `down` callback terminates. Any other functions, such as\nfunctions provided by `Ecto.Repo`, will be executed immediately unless they\nare called from within an anonymous function passed to `execute/1`.\n\nIn some situations you may want to guarantee that all of the previous steps\nhave been executed before continuing. This is useful when you need to apply a\nset of changes to the table before continuing with the migration. This can be\ndone with `flush/0`:\n\n    def up do\n      ...\n      flush()\n      ...\n    end\n\nHowever `flush/0` will raise if it would be called from `change` function when doing a rollback.\nTo avoid that we recommend to use `execute/2` with anonymous functions instead.\nFor more information and example usage please take a look at `execute/2` function.","ref":"Ecto.Migration.html#module-executing-and-flushing"},{"type":"behaviour","title":"Repo configuration - Ecto.Migration","doc":"","ref":"Ecto.Migration.html#module-repo-configuration"},{"type":"behaviour","title":"Migrator configuration - Ecto.Migration","doc":"These options configure where Ecto stores and how Ecto runs your migrations:\n\n  * `:migration_source` - Version numbers of migrations will be saved in a\n    table named `schema_migrations` by default. You can configure the name of\n    the table via:\n\n        config :app, App.Repo, migration_source: \"my_migrations\"\n\n  * `:migration_lock` - By default, Ecto will lock the migration source to throttle\n    multiple nodes to run migrations one at a time. You can disable the `migration_lock`\n    by setting it to `false`. You may also select a different locking strategy if\n    supported by the adapter. See the adapter docs for more information.\n\n        config :app, App.Repo, migration_lock: false\n\n        # Or use a different locking strategy. For example, Postgres can use advisory\n        # locks but be aware that your database configuration might not make this a good\n        # fit. See the Ecto.Adapters.Postgres for more information:\n        config :app, App.Repo, migration_lock: :pg_advisory_lock\n\n  * `:migration_repo` - The migration repository is where the table managing the\n    migrations will be stored (`migration_source` defines the table name). It defaults\n    to the given repository itself but you can configure it via:\n\n        config :app, App.Repo, migration_repo: App.MigrationRepo\n\n  * `:migration_cast_version_column` - Ecto uses a `version` column of type\n    `bigint` for the underlying migrations table (usually `schema_migrations`). By\n    default, Ecto doesn't cast this to a different type when reading or writing to\n    the database when running migrations. However, some web frameworks store this\n    column as a string. For compatibility reasons, you can set this option to `true`,\n    which makes Ecto perform a `CAST(version AS int)`. This used to be the default\n    behavior up to Ecto 3.10, so if you are upgrading to 3.11+ and want to keep the\n    old behavior, set this option to `true`.\n\n  * `:priv` - the priv directory for the repo with the location of important assets,\n    such as migrations. For a repository named `MyApp.FooRepo`, `:priv` defaults to\n    \"priv/foo_repo\" and migrations should be placed at \"priv/foo_repo/migrations\"\n\n  * `:start_apps_before_migration` - A list of applications to be started before\n    running migrations. Used by `Ecto.Migrator.with_repo/3` and the migration tasks:\n\n        config :app, App.Repo, start_apps_before_migration: [:ssl, :some_custom_logger]","ref":"Ecto.Migration.html#module-migrator-configuration"},{"type":"behaviour","title":"Migrations configuration - Ecto.Migration","doc":"These options configure the default values used by migrations. **It is generally\ndiscouraged to change any of those configurations after your database is deployed\nto production, as changing these options will retroactively change how all\nmigrations work**.\n\n  * `:migration_primary_key` - By default, Ecto uses the `:id` column with type\n    `:bigserial`, but you can configure it via:\n\n        config :app, App.Repo, migration_primary_key: [name: :uuid, type: :binary_id]\n\n        config :app, App.Repo, migration_primary_key: false\n\n    For Postgres version >= 10 `:identity` key may be used.\n    By default, all :identity column will be bigints. You may provide optional\n    parameters for `:start_value` and `:increment` to customize the created\n    sequence. Config example:\n\n        config :app, App.Repo, migration_primary_key: [type: :identity]\n\n  * `:migration_foreign_key` - By default, Ecto uses the `primary_key` type\n    for foreign keys when `references/2` is used, but you can configure it via:\n\n        config :app, App.Repo, migration_foreign_key: [column: :uuid, type: :binary_id]\n\n  * `:migration_timestamps` - By default, Ecto uses the `:naive_datetime` as the type,\n    `:inserted_at` as the name of the column for storing insertion times, `:updated_at` as\n    the name of the column for storing last-updated-at times, but you can configure it\n    via:\n\n        config :app, App.Repo, migration_timestamps: [\n          type: :utc_datetime,\n          inserted_at: :created_at,\n          updated_at: :changed_at\n        ]\n\n  * `:migration_default_prefix` - Ecto defaults to `nil` for the database prefix for\n    migrations, but you can configure it via:\n\n        config :app, App.Repo, migration_default_prefix: \"my_prefix\"","ref":"Ecto.Migration.html#module-migrations-configuration"},{"type":"behaviour","title":"Comments - Ecto.Migration","doc":"Migrations where you create or alter a table support specifying table\nand column comments. The same can be done when creating constraints\nand indexes. Not all databases support this feature.\n\n    def up do\n      create index(\"posts\", [:name], comment: \"Index Comment\")\n      create constraint(\"products\", \"price_must_be_positive\", check: \"price > 0\", comment: \"Constraint Comment\")\n      create table(\"weather\", prefix: \"north_america\", comment: \"Table Comment\") do\n        add :city, :string, size: 40, comment: \"Column Comment\"\n        timestamps()\n      end\n    end","ref":"Ecto.Migration.html#module-comments"},{"type":"behaviour","title":"Prefixes - Ecto.Migration","doc":"Migrations support specifying a table prefix or index prefix which will\ntarget either a schema (if using PostgreSQL) or a different database (if using\nMySQL). If no prefix is provided, the default schema or database is used.\n\nAny reference declared in the table migration refers by default to the table\nwith the same declared prefix. The prefix is specified in the table options:\n\n    def up do\n      create table(\"weather\", prefix: \"north_america\") do\n        add :city,    :string, size: 40\n        add :temp_lo, :integer\n        add :temp_hi, :integer\n        add :prcp,    :float\n        add :group_id, references(:groups)\n\n        timestamps()\n      end\n\n      create index(\"weather\", [:city], prefix: \"north_america\")\n    end\n\nNote: if using MySQL with a prefixed table, you must use the same prefix\nfor the references since cross-database references are not supported.\n\nWhen using a prefixed table with either MySQL or PostgreSQL, you must use the\nsame prefix for the index field to ensure that you index the prefix-qualified\ntable.","ref":"Ecto.Migration.html#module-prefixes"},{"type":"behaviour","title":"Transaction Callbacks - Ecto.Migration","doc":"If possible, each migration runs inside a transaction. This is true for Postgres,\nbut not true for MySQL, as the latter does not support DDL transactions.\n\nIn some rare cases, you may need to execute some common behavior after beginning\na migration transaction, or before committing that transaction. For instance, one\nmight desire to set a `lock_timeout` for each lock in the migration transaction.\n\nYou can do so by defining `c:after_begin/0` and `c:before_commit/0` callbacks to\nyour migration.\n\nHowever, if you need do so for every migration module, implement this callback\nfor every migration can be quite repetitive. Luckily, you can handle this by\nproviding your migration module:\n\n    defmodule MyApp.Migration do\n      defmacro __using__(_) do\n        quote do\n          use Ecto.Migration\n\n          def after_begin() do\n            repo().query! \"SET lock_timeout TO '5s'\"\n          end\n        end\n      end\n    end\n\nThen in your migrations you can `use MyApp.Migration` to share this behavior\namong all your migrations.","ref":"Ecto.Migration.html#module-transaction-callbacks"},{"type":"behaviour","title":"Additional resources - Ecto.Migration","doc":"* The [Safe Ecto Migrations guide](https://fly.io/phoenix-files/safe-ecto-migrations/)","ref":"Ecto.Migration.html#module-additional-resources"},{"type":"function","title":"Ecto.Migration.add/3","doc":"Adds a column when creating or altering a table.\n\nThis function also accepts Ecto primitive types as column types\nthat are normalized by the database adapter. For example,\n`:string` is converted to `:varchar`, `:binary` to `:bits` or `:blob`,\nand so on.\n\nHowever, the column type is not always the same as the type used in your\nschema. For example, a schema that has a `:string` field can be supported by\ncolumns of type `:char`, `:varchar`, `:text`, and others. For this reason,\nthis function also accepts `:text` and other type annotations that are native\nto the database. These are passed to the database as-is.\n\nTo sum up, the column type may be either an Ecto primitive type,\nwhich is normalized in cases where the database does not understand it,\nsuch as `:string` or `:binary`, or a database type which is passed as-is.\nCustom Ecto types like `Ecto.UUID` are not supported because\nthey are application-level concerns and may not always map to the database.\n\nNote: It may be necessary to quote case-sensitive, user-defined type names.\nFor example, PostgreSQL normalizes all identifiers to lower case unless\nthey are wrapped in double quotes. To ensure a case-sensitive type name\nis sent properly, it must be defined `:'\"LikeThis\"'` or `:\"\"LikeThis\"\"`.\nThis is not necessary for column names because Ecto quotes them automatically.\nType names are not automatically quoted because they may be expressions such\nas `varchar(255)`.","ref":"Ecto.Migration.html#add/3"},{"type":"function","title":"Examples - Ecto.Migration.add/3","doc":"create table(\"posts\") do\n      add :title, :string, default: \"Untitled\"\n    end\n\n    alter table(\"posts\") do\n      add :summary, :text               # Database type\n      add :object,  :map                # Elixir type which is handled by the database\n      add :custom, :'\"UserDefinedType\"' # A case-sensitive, user-defined type name\n      add :identity, :integer, generated: \"BY DEFAULT AS IDENTITY\" # Postgres generated identity column\n      add :generated_psql, :string, generated: \"ALWAYS AS (id::text) STORED\" # Postgres calculated column\n      add :generated_other, :string, generated: \"CAST(id AS char)\" # MySQL and TDS calculated column\n    end","ref":"Ecto.Migration.html#add/3-examples"},{"type":"function","title":"Options - Ecto.Migration.add/3","doc":"* `:primary_key` - when `true`, marks this field as the primary key.\n    If multiple fields are marked, a composite primary key will be created.\n  * `:default` - the column's default value. It can be a string, number, empty\n    list, list of strings, list of numbers, or a fragment generated by\n    `fragment/1`.\n  * `:null` - determines whether the column accepts null values. When not specified,\n    the database will use its default behaviour (which is to treat the column as nullable\n    in most databases).\n  * `:size` - the size of the type (for example, the number of characters).\n    The default is no size, except for `:string`, which defaults to `255`.\n  * `:precision` - the precision for a numeric type. Required when `:scale` is\n    specified.\n  * `:scale` - the scale of a numeric type. Defaults to `0`.\n  * `:comment` - adds a comment to the added column.\n  * `:after` - positions field after the specified one. Only supported on MySQL,\n    it is ignored by other databases.\n  * `:generated` - a string representing the expression for a generated column. See\n    above for a comprehensive set of examples for each of the built-in adapters. If\n    specified alongside `:start_value`/`:increment`, those options will be ignored.\n  * `:start_value` - option for `:identity` key, represents initial value in sequence\n    generation. Default is defined by the database.\n  * `:increment` - option for `:identity` key, represents increment value for\n    sequence generation. Default is defined by the database.\n  * `:fields` - option for `:duration` type. Restricts the set of stored interval fields\n    in the database.","ref":"Ecto.Migration.html#add/3-options"},{"type":"function","title":"Ecto.Migration.add_if_not_exists/3","doc":"Adds a column if it does not exist yet when altering a table.\n\nIf the `type` value is a `%Reference{}`, it is used to add a constraint.\n\n`type` and `opts` are exactly the same as in `add/3`.\n\nThis command is not reversible as Ecto does not know about column existence before the creation attempt.","ref":"Ecto.Migration.html#add_if_not_exists/3"},{"type":"function","title":"Examples - Ecto.Migration.add_if_not_exists/3","doc":"alter table(\"posts\") do\n      add_if_not_exists :title, :string, default: \"\"\n    end","ref":"Ecto.Migration.html#add_if_not_exists/3-examples"},{"type":"callback","title":"Ecto.Migration.after_begin/0","doc":"Migration code to run immediately after the transaction is opened.\n\nKeep in mind that it is treated like any normal migration code, and should\nconsider both the up *and* down cases of the migration.","ref":"Ecto.Migration.html#c:after_begin/0"},{"type":"macro","title":"Ecto.Migration.alter/2","doc":"Alters a table.","ref":"Ecto.Migration.html#alter/2"},{"type":"macro","title":"Examples - Ecto.Migration.alter/2","doc":"alter table(\"posts\") do\n      add :summary, :text\n      modify :title, :text\n      remove :views\n    end","ref":"Ecto.Migration.html#alter/2-examples"},{"type":"callback","title":"Ecto.Migration.before_commit/0","doc":"Migration code to run immediately before the transaction is closed.\n\nKeep in mind that it is treated like any normal migration code, and should\nconsider both the up *and* down cases of the migration.","ref":"Ecto.Migration.html#c:before_commit/0"},{"type":"function","title":"Ecto.Migration.constraint/3","doc":"Defines a constraint (either a check constraint or an exclusion constraint)\nto be evaluated by the database when a row is inserted or updated.","ref":"Ecto.Migration.html#constraint/3"},{"type":"function","title":"Examples - Ecto.Migration.constraint/3","doc":"create constraint(\"users\", :price_must_be_positive, check: \"price > 0\")\n    create constraint(\"size_ranges\", :no_overlap, exclude: ~s|gist (int4range(\"from\", \"to\", '[]') WITH &&)|)\n    drop   constraint(\"products\", \"price_must_be_positive\")","ref":"Ecto.Migration.html#constraint/3-examples"},{"type":"function","title":"Options - Ecto.Migration.constraint/3","doc":"* `:check` - A check constraint expression. Required when creating a check constraint.\n  * `:exclude` - An exclusion constraint expression. Required when creating an exclusion constraint.\n  * `:prefix` - The prefix for the table.\n  * `:validate` - Whether or not to validate the constraint on creation (true by default). See the section below for more information\n  * `:comment` - adds a comment to the constraint.","ref":"Ecto.Migration.html#constraint/3-options"},{"type":"function","title":"Using `validate: false` - Ecto.Migration.constraint/3","doc":"Validation/Enforcement of a constraint is enabled by default, but disabling on constraint\ncreation is supported by PostgreSQL, and MySQL, and can be done by setting `validate: false`.\n\nSetting `validate: false` as an option can be useful, as the creation of a constraint will cause\na full table scan to check existing rows. The constraint will still be enforced for subsequent\ninserts and updates, but should then be updated in a following command or migration to enforce\nthe new constraint.\n\nValidating / Enforcing the constraint in a later command, or migration, can be done like so:\n\n```\n  def change do\n    # PostgreSQL\n    execute \"ALTER TABLE products VALIDATE CONSTRAINT price_must_be_positive\", \"\"\n\n    # MySQL\n    execute \"ALTER TABLE products ALTER CONSTRAINT price_must_be_positive ENFORCED\", \"\"\n  end\n```\n\nSee the [Safe Ecto Migrations guide](https://fly.io/phoenix-files/safe-ecto-migrations/) for an\nin-depth explanation of the benefits of this approach.","ref":"Ecto.Migration.html#constraint/3-using-validate-false"},{"type":"function","title":"Ecto.Migration.create/1","doc":"Creates one of the following:\n\n  * an index\n  * a table with only the :id primary key\n  * a constraint\n\nWhen reversing (in a `change/0` running backwards), indexes are only dropped\nif they exist, and no errors are raised. To enforce dropping an index, use\n`drop/1`.","ref":"Ecto.Migration.html#create/1"},{"type":"function","title":"Examples - Ecto.Migration.create/1","doc":"create index(\"posts\", [:name])\n    create table(\"version\")\n    create constraint(\"products\", \"price_must_be_positive\", check: \"price > 0\")","ref":"Ecto.Migration.html#create/1-examples"},{"type":"macro","title":"Ecto.Migration.create/2","doc":"Creates a table.\n\nBy default, the table will also include an `:id` primary key field that\nhas a type of `:bigserial`. Check the `table/2` docs for more information.","ref":"Ecto.Migration.html#create/2"},{"type":"macro","title":"Examples - Ecto.Migration.create/2","doc":"create table(:posts) do\n      add :title, :string, default: \"Untitled\"\n      add :body,  :text\n\n      timestamps()\n    end","ref":"Ecto.Migration.html#create/2-examples"},{"type":"function","title":"Ecto.Migration.create_if_not_exists/1","doc":"Creates an index or a table with only `:id` field if one does not yet exist.","ref":"Ecto.Migration.html#create_if_not_exists/1"},{"type":"function","title":"Examples - Ecto.Migration.create_if_not_exists/1","doc":"create_if_not_exists index(\"posts\", [:name])\n\n    create_if_not_exists table(\"version\")","ref":"Ecto.Migration.html#create_if_not_exists/1-examples"},{"type":"macro","title":"Ecto.Migration.create_if_not_exists/2","doc":"Creates a table if it does not exist.\n\nWorks just like `create/2` but does not raise an error when the table\nalready exists.","ref":"Ecto.Migration.html#create_if_not_exists/2"},{"type":"function","title":"Ecto.Migration.direction/0","doc":"Gets the migrator direction.","ref":"Ecto.Migration.html#direction/0"},{"type":"function","title":"Ecto.Migration.drop/2","doc":"Drops one of the following:\n\n  * an index\n  * a table\n  * a constraint","ref":"Ecto.Migration.html#drop/2"},{"type":"function","title":"Examples - Ecto.Migration.drop/2","doc":"drop index(\"posts\", [:name])\n    drop table(\"posts\")\n    drop constraint(\"products\", \"price_must_be_positive\")\n    drop index(\"posts\", [:name]), mode: :cascade\n    drop table(\"posts\"), mode: :cascade","ref":"Ecto.Migration.html#drop/2-examples"},{"type":"function","title":"Options - Ecto.Migration.drop/2","doc":"* `:mode` - when set to `:cascade`, automatically drop objects that depend\n    on the index, and in turn all objects that depend on those objects\n    on the table. Default is `:restrict`","ref":"Ecto.Migration.html#drop/2-options"},{"type":"function","title":"Ecto.Migration.drop_if_exists/2","doc":"Drops a table or index if it exists.\n\nDoes not raise an error if the specified table or index does not exist.","ref":"Ecto.Migration.html#drop_if_exists/2"},{"type":"function","title":"Examples - Ecto.Migration.drop_if_exists/2","doc":"drop_if_exists index(\"posts\", [:name])\n    drop_if_exists table(\"posts\")\n    drop_if_exists index(\"posts\", [:name]), mode: :cascade\n    drop_if_exists table(\"posts\"), mode: :cascade","ref":"Ecto.Migration.html#drop_if_exists/2-examples"},{"type":"function","title":"Options - Ecto.Migration.drop_if_exists/2","doc":"* `:mode` - when set to `:cascade`, automatically drop objects that depend\n    on the index, and in turn all objects that depend on those objects\n    on the table. Default is `:restrict`","ref":"Ecto.Migration.html#drop_if_exists/2-options"},{"type":"function","title":"Ecto.Migration.execute/1","doc":"Executes arbitrary SQL, anonymous function or a keyword command.\n\nThe argument is typically a string, containing the SQL command to be executed.\nKeyword commands exist for non-SQL adapters and are not used in most\nsituations.\n\nYou may instead run arbitrary code as part of your migration by supplying an\nanonymous function. This defers execution of the anonymous function until\nthe migration callback has terminated (see [Executing and\nflushing](#module-executing-and-flushing)). This is most often used in\ncombination with `repo/0` by library authors who want to create high-level\nmigration helpers.\n\nReversible commands can be defined by calling `execute/2`.","ref":"Ecto.Migration.html#execute/1"},{"type":"function","title":"Examples - Ecto.Migration.execute/1","doc":"execute \"CREATE EXTENSION postgres_fdw\"\n\n    execute create: \"posts\", capped: true, size: 1024\n\n    execute(fn -> repo().query!(\"SELECT $1::integer + $2\", [40, 2], [log: :info]) end)\n\n    execute(fn -> repo().update_all(\"posts\", set: [published: true]) end)","ref":"Ecto.Migration.html#execute/1-examples"},{"type":"function","title":"Ecto.Migration.execute/2","doc":"Executes reversible SQL commands.\n\nThis is useful for database-specific functionality that does not\nwarrant special support in Ecto, for example, creating and dropping\na PostgreSQL extension. The `execute/2` form avoids having to define\nseparate `up/0` and `down/0` blocks that each contain an `execute/1`\nexpression.\n\nThe allowed parameters are explained in `execute/1`.","ref":"Ecto.Migration.html#execute/2"},{"type":"function","title":"Examples - Ecto.Migration.execute/2","doc":"defmodule MyApp.MyMigration do\n      use Ecto.Migration\n\n      def change do\n        execute \"CREATE EXTENSION postgres_fdw\", \"DROP EXTENSION postgres_fdw\"\n        execute(&execute_up/0, &execute_down/0)\n      end\n\n      defp execute_up, do: repo().query!(\"select 'Up query …';\", [], [log: :info])\n      defp execute_down, do: repo().query!(\"select 'Down query …';\", [], [log: :info])\n    end","ref":"Ecto.Migration.html#execute/2-examples"},{"type":"function","title":"Ecto.Migration.execute_file/1","doc":"Executes a SQL command from a file.\n\nThe argument must be a path to a file containing a SQL command.\n\nReversible commands can be defined by calling `execute_file/2`.","ref":"Ecto.Migration.html#execute_file/1"},{"type":"function","title":"Ecto.Migration.execute_file/2","doc":"Executes reversible SQL commands from files.\n\nEach argument must be a path to a file containing a SQL command.\n\nSee `execute/2` for more information on executing SQL commands.","ref":"Ecto.Migration.html#execute_file/2"},{"type":"macro","title":"Ecto.Migration.flush/0","doc":"Executes queue migration commands.","ref":"Ecto.Migration.html#flush/0"},{"type":"function","title":"Ecto.Migration.fragment/1","doc":"Generates a fragment to be used as a default value.","ref":"Ecto.Migration.html#fragment/1"},{"type":"function","title":"Examples - Ecto.Migration.fragment/1","doc":"create table(\"posts\") do\n      add :inserted_at, :naive_datetime, default: fragment(\"now()\")\n    end","ref":"Ecto.Migration.html#fragment/1-examples"},{"type":"function","title":"Ecto.Migration.index/3","doc":"Returns an index struct that can be given to `create/1`, `drop/1`, etc.\n\nExpects the table name as the first argument and the index field(s) as\nthe second. The fields can be atoms, representing columns, or strings,\nrepresenting expressions that are sent as-is to the database.","ref":"Ecto.Migration.html#index/3"},{"type":"function","title":"Options - Ecto.Migration.index/3","doc":"* `:name` - the name of the index. Defaults to \"#{table}_#{column}_index\".\n  * `:prefix` - specify an optional prefix for the index.\n  * `:unique` - indicates whether the index should be unique. Defaults to `false`.\n  * `:comment` - adds a comment to the index.\n  * `:using` - configures the index type.\n\nSome options are supported only by some databases:\n\n  * `:concurrently` - indicates whether the index should be created/dropped\n    concurrently in MSSQL and PostgreSQL.\n  * `:include` - specify fields for a covering index,\n    [supported by PostgreSQL only](https://www.postgresql.org/docs/current/indexes-index-only-scans.html).\n  * `:nulls_distinct` - specify whether null values should be considered\n    distinct for a unique index. Defaults to `nil`, which will not add the\n    parameter to the generated SQL and thus use the database default.\n    This option is currently only supported by PostgreSQL 15+.\n    For MySQL, it is always false. For MSSQL, it is always true.\n    See the dedicated section on this option for more information.\n  * `:only` - indicates to not recurse creating indexes on partitions.\n    [supported by PostgreSQL only](https://www.postgresql.org/docs/current/ddl-partitioning.html#DDL-PARTITIONING-DECLARATIVE-MAINTENANCE).\n  * `:options` - configures index options (WITH clause) for both PostgreSQL\n    and MSSQL\n  * `:where` - specify conditions for a partial index (PostgreSQL) /\n    filtered index (MSSQL).","ref":"Ecto.Migration.html#index/3-options"},{"type":"function","title":"Adding/dropping indexes concurrently - Ecto.Migration.index/3","doc":"PostgreSQL supports adding/dropping indexes concurrently (see the\n[docs](http://www.postgresql.org/docs/current/static/sql-createindex.html)).\nHowever, this feature does not work well with the transactions used by\nEcto to guarantee integrity during migrations.\n\nYou can address this with two changes:\n\n  1. Change your repository to use PG advisory locks as the migration lock.\n     Note this may not be supported by Postgres-like databases and proxies.\n\n  2. Disable DDL transactions. Doing this removes the guarantee that all of\n    the changes in the migration will happen at once, so you will want to\n    keep it short.\n\nIf the database adapter supports several migration lock strategies, such as\nPostgrex, then review those strategies and consider using a strategy that\nutilizes advisory locks to facilitate running migrations one at a time even\nacross multiple nodes. For example:","ref":"Ecto.Migration.html#index/3-adding-dropping-indexes-concurrently"},{"type":"function","title":"Config file (PostgreSQL) - Ecto.Migration.index/3","doc":"config MyApp.Repo, migration_lock: :pg_advisory_lock","ref":"Ecto.Migration.html#index/3-config-file-postgresql"},{"type":"function","title":"Migration file - Ecto.Migration.index/3","doc":"defmodule MyRepo.Migrations.CreateIndexes do\n      use Ecto.Migration\n      @disable_ddl_transaction true\n\n      def change do\n        create index(\"posts\", [:slug], concurrently: true)\n      end\n    end\n\nAlternately, you can add `@disable_migration_lock true` to your migration file.\nThis would mean that different nodes in a multi-node setup could run the same\nmigration at once. It is recommended to isolate your migrations to a single node\nwhen using concurrent index creation without an advisory lock.","ref":"Ecto.Migration.html#index/3-migration-file"},{"type":"function","title":"Index types - Ecto.Migration.index/3","doc":"When creating an index, the index type can be specified with the `:using`\noption. The `:using` option can be an atom or a string, and its value is\npassed to the generated `USING` clause as-is.\n\nFor example, PostgreSQL supports several index types like B-tree (the\ndefault), Hash, GIN, and GiST. More information on index types can be found\nin the [PostgreSQL docs](http://www.postgresql.org/docs/current/indexes-types.html).","ref":"Ecto.Migration.html#index/3-index-types"},{"type":"function","title":"Partial indexes - Ecto.Migration.index/3","doc":"Databases like PostgreSQL and MSSQL support partial indexes.\n\nA partial index is an index built over a subset of a table. The subset\nis defined by a conditional expression using the `:where` option.\nThe `:where` option can be an atom or a string; its value is passed\nto the generated `WHERE` clause as-is.\n\nMore information on partial indexes can be found in the [PostgreSQL\ndocs](http://www.postgresql.org/docs/current/indexes-partial.html).","ref":"Ecto.Migration.html#index/3-partial-indexes"},{"type":"function","title":"The `:nulls_distinct` option - Ecto.Migration.index/3","doc":"A unique index does not prevent multiple null values by default in most databases.\n\nFor example, imagine we have a \"products\" table and need to guarantee that\nsku's are unique within their category, but the category is optional.\nCreating a regular unique index over the sku and category_id fields with:\n\n    create index(\"products\", [:sku, :category_id], unique: true)\n\nwill allow products with the same sku to be inserted if their category_id is `nil`.\nThe `:nulls_distinct` option can be used to change this behavior by considering\nnull values as equal, i.e. not distinct:\n\n    create index(\"products\", [:sku, :category_id], unique: true, nulls_distinct: false)\n\nThis option is currently only supported by PostgreSQL 15+.\nAs a workaround for older PostgreSQL versions and other databases, an\nadditional partial unique index for the sku can be created:\n\n    create index(\"products\", [:sku, :category_id], unique: true)\n    create index(\"products\", [:sku], unique: true, where: \"category_id IS NULL\")","ref":"Ecto.Migration.html#index/3-the-nulls_distinct-option"},{"type":"function","title":"Examples - Ecto.Migration.index/3","doc":"# With no name provided, the name of the below index defaults to\n    # products_category_id_sku_index\n    create index(\"products\", [:category_id, :sku], unique: true)\n\n    # The name can also be set explicitly\n    create index(\"products\", [:category_id, :sku], name: :my_special_name)\n\n    # Indexes can be added concurrently\n    create index(\"products\", [:category_id, :sku], concurrently: true)\n\n    # The index type can be specified\n    create index(\"products\", [:name], using: :hash)\n\n    # Partial indexes are created by specifying a :where option\n    create index(\"products\", [:user_id], where: \"price = 0\", name: :free_products_index)\n\n    # Covering indexes are created by specifying a :include option\n    create index(\"products\", [:user_id], include: [:category_id])\n\nIndexes also support custom expressions. Some databases may require the\nindex expression to be written between parentheses:\n\n    # Create an index on a custom expression\n    create index(\"products\", [\"(lower(name))\"], name: :products_lower_name_index)\n\n    # Create a tsvector GIN index on PostgreSQL\n    create index(\"products\", [\"(to_tsvector('english', name))\"],\n                 name: :products_name_vector, using: \"GIN\")\n\nIf the expression is a column name, it will not be quoted. This may cause issues\nwhen the column is named after a reserved word. Consider using an atom instead.\nFor example, the name `offset` is reserved in many databases so the following\ncould produce an error: `create index(\"products\", [\"offset\"])`.","ref":"Ecto.Migration.html#index/3-examples"},{"type":"function","title":"Ecto.Migration.modify/3","doc":"Modifies the type of a column when altering a table.\n\nThis command is not reversible unless the `:from` option is provided.\nWhen the `:from` option is set, the adapter will try to drop\nthe corresponding foreign key constraints before modifying the type.\nGenerally speaking, you want to pass the type and each option\nyou are modifying to `:from`, so the column can be rolled back properly.\nHowever, note that `:from` cannot be used to modify primary keys,\nas those are generally trickier to revert.\n\nSee `add/3` for more information on supported types.\n\nIf you want to modify a column without changing its type,\nsuch as adding or dropping a null constraints, consider using\nthe `execute/2` command with the relevant SQL command instead\nof `modify/3`, if supported by your database. This may avoid\nredundant type updates and be more efficient, as an unnecessary\ntype update can lock the table, even if the type actually\ndoesn't change.","ref":"Ecto.Migration.html#modify/3"},{"type":"function","title":"Examples - Ecto.Migration.modify/3","doc":"alter table(\"posts\") do\n      modify :title, :text\n    end\n\n    # Self rollback when using the :from option\n    alter table(\"posts\") do\n      modify :title, :text, from: :string\n    end\n\n    # Modify column with rollback options\n    alter table(\"posts\") do\n      modify :title, :text, null: false, from: {:string, null: true}\n    end\n\n    # Modify the :on_delete option of an existing foreign key\n    alter table(\"comments\") do\n      modify :post_id, references(:posts, on_delete: :delete_all),\n        from: references(:posts, on_delete: :nothing)\n    end","ref":"Ecto.Migration.html#modify/3-examples"},{"type":"function","title":"Options - Ecto.Migration.modify/3","doc":"* `:null` - determines whether the column accepts null values. If this option is\n    not set, the nullable behaviour of the underlying column is not modified.\n  * `:default` - changes the default value of the column.\n  * `:from` - specifies the current type and options of the column.\n  * `:size` - specifies the size of the type (for example, the number of characters).\n    The default is no size.\n  * `:precision` - the precision for a numeric type. Required when `:scale` is\n    specified.\n  * `:scale` - the scale of a numeric type. Defaults to `0`.\n  * `:comment` - adds a comment to the modified column.","ref":"Ecto.Migration.html#modify/3-options"},{"type":"function","title":"Ecto.Migration.prefix/0","doc":"Gets the migrator prefix.","ref":"Ecto.Migration.html#prefix/0"},{"type":"function","title":"Ecto.Migration.references/2","doc":"Defines a foreign key.\n\nBy default it assumes you are linking to the referenced table\nvia its primary key with name `:id`. If you are using a non-default\nkey setup (e.g. using `uuid` type keys) you must ensure you set the\noptions, such as `:column` and `:type`, to match your target key.","ref":"Ecto.Migration.html#references/2"},{"type":"function","title":"Examples - Ecto.Migration.references/2","doc":"create table(\"products\") do\n      add :group_id, references(\"groups\")\n    end\n\n    create table(\"categories\") do\n      add :group_id, :integer\n      # A composite foreign that points from categories (product_id, group_id)\n      # to products (id, group_id)\n      add :product_id, references(\"products\", with: [group_id: :group_id])\n    end","ref":"Ecto.Migration.html#references/2-examples"},{"type":"function","title":"Options - Ecto.Migration.references/2","doc":"* `:name` - The name of the underlying reference, which defaults to\n    \"#{table}_#{column}_fkey\".\n  * `:column` - The column name in the referenced table, which defaults to `:id`.\n  * `:prefix` - The prefix for the reference. Defaults to the prefix\n    defined by the block's `table/2` struct (the \"products\" table in\n    the example above), or `nil`.\n  * `:type` - The foreign key type, which defaults to `:bigserial`.\n  * `:on_delete` - What to do if the referenced entry is deleted. May be\n    `:nothing` (default), `:delete_all`, `:nilify_all`, `{:nilify, columns}`,\n    or `:restrict`. `{:nilify, columns}` expects a list of atoms for `columns`\n    and is not supported by all databases.\n  * `:on_update` - What to do if the referenced entry is updated. May be\n    `:nothing` (default), `:update_all`, `:nilify_all`, or `:restrict`.\n  * `:validate` - Whether or not to validate the foreign key constraint on\n     creation or not. Only available in PostgreSQL, and should be followed by\n     a command to validate the foreign key in a following migration if false.\n  * `:with` - defines additional keys to the foreign key in order to build\n    a composite foreign key\n  * `:match` - select if the match is `:simple`, `:partial`, or `:full`. This is\n    [supported only by PostgreSQL](https://www.postgresql.org/docs/current/sql-createtable.html)\n    at the moment.","ref":"Ecto.Migration.html#references/2-options"},{"type":"function","title":"Ecto.Migration.remove/1","doc":"Removes a column when altering a table.\n\nThis command is not reversible as Ecto does not know what type it should add\nthe column back as. See `remove/3` as a reversible alternative.","ref":"Ecto.Migration.html#remove/1"},{"type":"function","title":"Examples - Ecto.Migration.remove/1","doc":"alter table(\"posts\") do\n      remove :title\n    end","ref":"Ecto.Migration.html#remove/1-examples"},{"type":"function","title":"Ecto.Migration.remove/3","doc":"Removes a column in a reversible way when altering a table.\n\n`type` and `opts` are exactly the same as in `add/3`, and\nthey are used when the command is reversed.\n\nIf the `type` value is a `%Reference{}`, it is used to remove the constraint.","ref":"Ecto.Migration.html#remove/3"},{"type":"function","title":"Examples - Ecto.Migration.remove/3","doc":"alter table(\"posts\") do\n      remove :title, :string, default: \"\"\n    end","ref":"Ecto.Migration.html#remove/3-examples"},{"type":"function","title":"Ecto.Migration.remove_if_exists/1","doc":"Removes a column if the column exists.\n\nThis command is not reversible as Ecto does not know whether or not the column existed before the removal attempt.","ref":"Ecto.Migration.html#remove_if_exists/1"},{"type":"function","title":"Examples - Ecto.Migration.remove_if_exists/1","doc":"alter table(\"posts\") do\n      remove_if_exists :title\n    end","ref":"Ecto.Migration.html#remove_if_exists/1-examples"},{"type":"function","title":"Ecto.Migration.remove_if_exists/2","doc":"Removes a column if the column exists.\n\nIf the type is a reference, removes the foreign key constraint for the reference first, if it exists.\n\nThis command is not reversible as Ecto does not know whether or not the column existed before the removal attempt.","ref":"Ecto.Migration.html#remove_if_exists/2"},{"type":"function","title":"Examples - Ecto.Migration.remove_if_exists/2","doc":"alter table(\"posts\") do\n      remove_if_exists :author_id, references(:authors)\n    end","ref":"Ecto.Migration.html#remove_if_exists/2-examples"},{"type":"function","title":"Ecto.Migration.rename/2","doc":"Renames a table or index.","ref":"Ecto.Migration.html#rename/2"},{"type":"function","title":"Examples - Ecto.Migration.rename/2","doc":"# rename a table\n    rename table(\"posts\"), to: table(\"new_posts\")\n\n    # rename an index\n    rename(index(:people, [:name], name: \"persons_name_index\"), to: \"people_name_index\")","ref":"Ecto.Migration.html#rename/2-examples"},{"type":"function","title":"Ecto.Migration.rename/3","doc":"Renames a column.\n\nNote that this occurs outside of the `alter` statement.","ref":"Ecto.Migration.html#rename/3"},{"type":"function","title":"Examples - Ecto.Migration.rename/3","doc":"rename table(\"posts\"), :title, to: :summary","ref":"Ecto.Migration.html#rename/3-examples"},{"type":"function","title":"Ecto.Migration.repo/0","doc":"Gets the migrator repo.","ref":"Ecto.Migration.html#repo/0"},{"type":"function","title":"Ecto.Migration.table/2","doc":"Returns a table struct that can be given to `create/2`, `alter/2`, `drop/1`,\netc.","ref":"Ecto.Migration.html#table/2"},{"type":"function","title":"Examples - Ecto.Migration.table/2","doc":"create table(\"products\") do\n      add :name, :string\n      add :price, :decimal\n    end\n\n    drop table(\"products\")\n\n    create table(\"products\", primary_key: false) do\n      add :name, :string\n      add :price, :decimal\n    end\n\n    create table(\"daily_prices\", primary_key: false, options: \"PARTITION BY RANGE (date)\") do\n      add :name, :string, primary_key: true\n      add :date, :date, primary_key: true\n      add :price, :decimal\n    end\n\n    create table(\"users\", primary_key: false) do\n      add :id, :identity, primary_key: true, start_value: 100, increment: 20\n    end","ref":"Ecto.Migration.html#table/2-examples"},{"type":"function","title":"Options - Ecto.Migration.table/2","doc":"* `:primary_key` - when `false`, a primary key field is not generated on table\n    creation. Alternatively, a keyword list in the same style of the\n    `:migration_primary_key` repository configuration can be supplied\n    to control the generation of the primary key field. The keyword list\n    must include `:name` and `:type`. See `add/3` for further options.\n  * `:engine` - customizes the table storage for supported databases. For MySQL,\n    the default is InnoDB.\n  * `:prefix` - the prefix for the table. This prefix will automatically be used\n    for all constraints and references defined for this table unless explicitly\n    overridden in said constraints/references.\n  * `:comment` - adds a comment to the table.\n  * `:options` - provide custom options that will be appended after the generated\n    statement. For example, \"WITH\", \"INHERITS\", or \"ON COMMIT\" clauses. \"PARTITION BY\"\n    can be provided for databases that support table partitioning.","ref":"Ecto.Migration.html#table/2-options"},{"type":"function","title":"Ecto.Migration.timestamps/1","doc":"Adds `:inserted_at` and `:updated_at` timestamp columns.\n\nThose columns are of `:naive_datetime` type and by default cannot be null. A\nlist of `opts` can be given to customize the generated fields.\n\nFollowing options will override the repo configuration specified by\n`:migration_timestamps` option.","ref":"Ecto.Migration.html#timestamps/1"},{"type":"function","title":"Options - Ecto.Migration.timestamps/1","doc":"* `:inserted_at` - the name of the column for storing insertion times.\n    Setting it to `false` disables the column.\n  * `:updated_at` - the name of the column for storing last-updated-at times.\n    Setting it to `false` disables the column.\n  * `:type` - the type of the `:inserted_at` and `:updated_at` columns.\n    Defaults to `:naive_datetime`.\n  * `:default` - the columns' default value. It can be a string, number, empty\n    list, list of strings, list of numbers, or a fragment generated by\n    `fragment/1`.","ref":"Ecto.Migration.html#timestamps/1-options"},{"type":"function","title":"Ecto.Migration.unique_index/3","doc":"Shortcut for creating a unique index.\n\nSee `index/3` for more information.","ref":"Ecto.Migration.html#unique_index/3"},{"type":"module","title":"Ecto.Migrator","doc":"Lower level API for managing migrations.\n\nEctoSQL provides three mix tasks for running and managing migrations:\n\n  * `mix ecto.migrate` - migrates a repository\n  * `mix ecto.rollback` - rolls back a particular migration\n  * `mix ecto.migrations` - shows all migrations and their status\n\nThose tasks are built on top of the functions in this module.\nWhile the tasks above cover most use cases, it may be necessary\nfrom time to time to jump into the lower level API. For example,\nif you are assembling an Elixir release, Mix is not available,\nso this module provides a nice complement to still migrate your\nsystem.\n\nTo learn more about migrations in general, see `Ecto.Migration`.","ref":"Ecto.Migrator.html"},{"type":"module","title":"Example: Running an individual migration - Ecto.Migrator","doc":"Imagine you have this migration:\n\n    defmodule MyApp.MigrationExample do\n      use Ecto.Migration\n\n      def up do\n        execute \"CREATE TABLE users(id serial PRIMARY_KEY, username text)\"\n      end\n\n      def down do\n        execute \"DROP TABLE users\"\n      end\n    end\n\nYou can execute it manually with:\n\n    Ecto.Migrator.up(Repo, 20080906120000, MyApp.MigrationExample)","ref":"Ecto.Migrator.html#module-example-running-an-individual-migration"},{"type":"module","title":"Example: Running migrations in a release - Ecto.Migrator","doc":"Elixir v1.9 introduces `mix release`, which generates a self-contained\ndirectory that consists of your application code, all of its dependencies,\nplus the whole Erlang Virtual Machine (VM) and runtime.\n\nWhen a release is assembled, Mix is no longer available inside a release\nand therefore none of the Mix tasks. Users may still need a mechanism to\nmigrate their databases. This can be achieved with using the `Ecto.Migrator`\nmodule:\n\n    defmodule MyApp.Release do\n      @app :my_app\n\n      def migrate do\n        for repo <- repos() do\n          {:ok, _, _} = Ecto.Migrator.with_repo(repo, &Ecto.Migrator.run(&1, :up, all: true))\n        end\n      end\n\n      def rollback(repo, version) do\n        {:ok, _, _} = Ecto.Migrator.with_repo(repo, &Ecto.Migrator.run(&1, :down, to: version))\n      end\n\n      defp repos do\n        Application.load(@app)\n        Application.fetch_env!(@app, :ecto_repos)\n      end\n    end\n\nThe example above uses `with_repo/3` to make sure the repository is\nstarted and then runs all migrations up or a given migration down.\nNote you will have to replace `MyApp` and `:my_app` on the first two\nlines by your actual application name. Once the file above is added\nto your application, you can assemble a new release and invoke the\ncommands above in the release root like this:\n\n    $ bin/my_app eval \"MyApp.Release.migrate\"\n    $ bin/my_app eval \"MyApp.Release.rollback(MyApp.Repo, 20190417140000)\"","ref":"Ecto.Migrator.html#module-example-running-migrations-in-a-release"},{"type":"module","title":"Example: Running migrations on application startup - Ecto.Migrator","doc":"Add the following to the top of your application children spec:\n\n    {Ecto.Migrator,\n     repos: Application.fetch_env!(:my_app, :ecto_repos),\n     skip: System.get_env(\"SKIP_MIGRATIONS\") == \"true\"}\n\nTo skip migrations you can also pass `skip: true` or as in the example\nset the environment variable `SKIP_MIGRATIONS` to a truthy value.\n\nAnd all other options described in `up/4` are allowed,\nfor example if you want to log the SQL commands,\nand run migrations in a prefix:\n\n    {Ecto.Migrator,\n     repos: Application.fetch_env!(:my_app, :ecto_repos),\n     log_migrator_sql: true,\n     prefix: \"my_app\"}\n\nTo roll back you'd do it normally:\n\n    $ mix ecto.rollback","ref":"Ecto.Migrator.html#module-example-running-migrations-on-application-startup"},{"type":"function","title":"Ecto.Migrator.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Ecto.Migrator.html#child_spec/1"},{"type":"function","title":"Ecto.Migrator.down/4","doc":"Runs a down migration on the given repository.","ref":"Ecto.Migrator.html#down/4"},{"type":"function","title":"Options - Ecto.Migrator.down/4","doc":"* `:log` - the level to use for logging of migration commands. Defaults to `:info`.\n    Can be any of `Logger.level/0` values or a boolean.\n  * `:log_migrations_sql` - the level to use for logging of SQL commands\n    generated by migrations. Can be any of the `Logger.level/0` values\n    or a boolean. If `false`, logging is disabled. If `true`, uses the configured\n    Repo logger level. Defaults to `false`\n  * `:log_migrator_sql` - the level to use for logging of SQL commands emitted\n    by the migrator, such as transactions, locks, etc. Can be any of the `Logger.level/0`\n    values or a boolean. If `false`, logging is disabled. If `true`, uses the configured\n    Repo logger level. Defaults to `false`\n  * `:prefix` - the prefix to run the migrations on\n  * `:dynamic_repo` - the name of the Repo supervisor process.\n    See `c:Ecto.Repo.put_dynamic_repo/1`.","ref":"Ecto.Migrator.html#down/4-options"},{"type":"function","title":"Ecto.Migrator.migrated_versions/2","doc":"Gets all migrated versions.\n\nThis function ensures the migration table exists\nif no table has been defined yet.","ref":"Ecto.Migrator.html#migrated_versions/2"},{"type":"function","title":"Options - Ecto.Migrator.migrated_versions/2","doc":"* `:prefix` - the prefix to run the migrations on\n  * `:dynamic_repo` - the name of the Repo supervisor process.\n    See `c:Ecto.Repo.put_dynamic_repo/1`.\n  * `:skip_table_creation` - skips any attempt to create the migration table\n    Useful for situations where user needs to check migrations but has\n    insufficient permissions to create the table.  Note that migrations\n    commands may fail if this is set to true. Defaults to `false`.  Accepts a\n    boolean.","ref":"Ecto.Migrator.html#migrated_versions/2-options"},{"type":"function","title":"Ecto.Migrator.migrations/1","doc":"Returns an array of tuples as the migration status of the given repo,\nwithout actually running any migrations.\n\nEquivalent to:\n\n    Ecto.Migrator.migrations(repo, [Ecto.Migrator.migrations_path(repo)])","ref":"Ecto.Migrator.html#migrations/1"},{"type":"function","title":"Ecto.Migrator.migrations/3","doc":"Returns an array of tuples as the migration status of the given repo,\nwithout actually running any migrations.","ref":"Ecto.Migrator.html#migrations/3"},{"type":"function","title":"Ecto.Migrator.migrations_path/2","doc":"Gets the migrations path from a repository.\n\nThis function accepts an optional second parameter to customize the\nmigrations directory. This can be used to specify a custom migrations\npath.","ref":"Ecto.Migrator.html#migrations_path/2"},{"type":"function","title":"Ecto.Migrator.run/3","doc":"Runs migrations for the given repository.\n\nEquivalent to:\n\n    Ecto.Migrator.run(repo, [Ecto.Migrator.migrations_path(repo)], direction, opts)\n\nSee `run/4` for more information.","ref":"Ecto.Migrator.html#run/3"},{"type":"function","title":"Ecto.Migrator.run/4","doc":"Apply migrations to a repository with a given strategy.\n\nThe second argument identifies where the migrations are sourced from.\nA binary representing directory (or a list of binaries representing\ndirectories) may be passed, in which case we will load all files\nfollowing the \"#{VERSION}_#{NAME}.exs\" schema. The `migration_source`\nmay also be a list of tuples that identify the version number and\nmigration modules to be run, for example:\n\n    Ecto.Migrator.run(Repo, [{0, MyApp.Migration1}, {1, MyApp.Migration2}, ...], :up, opts)\n\nA strategy (which is one of `:all`, `:step`, `:to`, or `:to_exclusive`) must be given as\nan option.","ref":"Ecto.Migrator.html#run/4"},{"type":"function","title":"Execution model - Ecto.Migrator.run/4","doc":"In order to run migrations, at least two database connections are\nnecessary. One is used to lock the \"schema_migrations\" table and\nthe other one to effectively run the migrations. This allows multiple\nnodes to run migrations at the same time, but guarantee that only one\nof them will effectively migrate the database.\n\nA downside of this approach is that migrations cannot run dynamically\nduring test under the `Ecto.Adapters.SQL.Sandbox`, as the sandbox has\nto share a single connection across processes to guarantee the changes\ncan be reverted.","ref":"Ecto.Migrator.html#run/4-execution-model"},{"type":"function","title":"Options - Ecto.Migrator.run/4","doc":"* `:all` - runs all available if `true`\n\n  * `:step` - runs the specific number of migrations\n\n  * `:to` - runs all until the supplied version is reached\n    (including the version given in `:to`)\n\n  * `:to_exclusive` - runs all until the supplied version is reached\n    (excluding the version given in `:to_exclusive`)\n\nPlus all other options described in `up/4`.","ref":"Ecto.Migrator.html#run/4-options"},{"type":"function","title":"Ecto.Migrator.start_link/1","doc":"Runs migrations as part of your supervision tree.","ref":"Ecto.Migrator.html#start_link/1"},{"type":"function","title":"Options - Ecto.Migrator.start_link/1","doc":"* `:repos` - Required option to tell the migrator which Repo's to\n    migrate. Example: `repos: [MyApp.Repo]`\n\n  * `:skip` - Option to skip migrations. Defaults to `false`.\n\nPlus all other options described in `up/4`.\n\nSee \"Example: Running migrations on application startup\" for more info.","ref":"Ecto.Migrator.html#start_link/1-options"},{"type":"function","title":"Ecto.Migrator.up/4","doc":"Runs an up migration on the given repository.","ref":"Ecto.Migrator.html#up/4"},{"type":"function","title":"Options - Ecto.Migrator.up/4","doc":"* `:log` - the level to use for logging of migration instructions.\n    Defaults to `:info`. Can be any of `Logger.level/0` values or a boolean.\n    If `false`, it also avoids logging messages from the database.\n  * `:log_migrations_sql` - the level to use for logging of SQL commands\n    generated by migrations. Can be any of the `Logger.level/0` values\n    or a boolean. If `false`, logging is disabled. If `true`, uses the configured\n    Repo logger level. Defaults to `false`\n  * `:log_migrator_sql` - the level to use for logging of SQL commands emitted\n    by the migrator, such as transactions, locks, etc. Can be any of the `Logger.level/0`\n    values or a boolean. If `false`, logging is disabled. If `true`, uses the configured\n    Repo logger level. Defaults to `false`\n  * `:prefix` - the prefix to run the migrations on\n  * `:dynamic_repo` - the name of the Repo supervisor process.\n    See `c:Ecto.Repo.put_dynamic_repo/1`.\n  * `:strict_version_order` - abort when applying a migration with old timestamp\n    (otherwise it emits a warning)","ref":"Ecto.Migrator.html#up/4-options"},{"type":"function","title":"Ecto.Migrator.with_repo/3","doc":"Ensures the repo is started to perform migration operations.\n\nAll of the application required to run the repo will be started\nbefore hand with chosen mode. If the repo has not yet been started,\nit is manually started, with a `:pool_size` of 2, before the given\nfunction is executed, and the repo is then terminated. If the repo\nwas already started, then the function is directly executed, without\nterminating the repo afterwards.\n\nAlthough this function was designed to start repositories for running\nmigrations, it can be used by any code, Mix task, or release tooling\nthat needs to briefly start a repository to perform a certain operation\nand then terminate.\n\nThe repo may also configure a `:start_apps_before_migration` option\nwhich is a list of applications to be started before the migration\nruns.\n\nIt returns `{:ok, fun_return, apps}`, with all apps that have been\nstarted, or `{:error, term}`.","ref":"Ecto.Migrator.html#with_repo/3"},{"type":"function","title":"Options - Ecto.Migrator.with_repo/3","doc":"* `:pool_size` - The pool size to start the repo for migrations.\n    Defaults to 2.\n  * `:mode` - The mode to start all applications.\n    Defaults to `:permanent`.","ref":"Ecto.Migrator.html#with_repo/3-options"},{"type":"function","title":"Examples - Ecto.Migrator.with_repo/3","doc":"{:ok, _, _} =\n      Ecto.Migrator.with_repo(repo, fn repo ->\n        Ecto.Migrator.run(repo, :up, all: true)\n      end)","ref":"Ecto.Migrator.html#with_repo/3-examples"},{"type":"task","title":"mix ecto.dump","doc":"Dumps the current environment's database structure for the\ngiven repository into a structure file.\n\nThe repository must be set under `:ecto_repos` in the\ncurrent app configuration or given via the `-r` option.\n\nThis task needs some shell utility to be present on the machine\nrunning the task.\n\n Database   | Utility needed\n :--------- | :-------------\n PostgreSQL | pg_dump\n MySQL      | mysqldump","ref":"Mix.Tasks.Ecto.Dump.html"},{"type":"task","title":"Example - mix ecto.dump","doc":"$ mix ecto.dump","ref":"Mix.Tasks.Ecto.Dump.html#module-example"},{"type":"task","title":"Command line options - mix ecto.dump","doc":"* `-r`, `--repo` - the repo to load the structure info from\n  * `-d`, `--dump-path` - the path of the dump file to create\n  * `-q`, `--quiet` - run the command quietly\n  * `--no-compile` - does not compile applications before dumping\n  * `--no-deps-check` - does not check dependencies before dumping\n  * `--prefix` - prefix that will be included in the structure dump.\n    Can include multiple prefixes (ex. `--prefix foo --prefix bar`) with\n    PostgreSQL but not MySQL. When specified, the prefixes will have\n    their definitions dumped along with the data in their migration table.\n    The default behavior is dependent on the adapter for backwards compatibility\n    reasons. For PostgreSQL, the configured database has the definitions dumped\n    from all of its schemas but only the data from the migration table\n    from the `public` schema is included. For MySQL, only the configured\n    database and its migration table are dumped.","ref":"Mix.Tasks.Ecto.Dump.html#module-command-line-options"},{"type":"task","title":"mix ecto.gen.migration","doc":"Generates a migration.\n\nThe repository must be set under `:ecto_repos` in the\ncurrent app configuration or given via the `-r` option.","ref":"Mix.Tasks.Ecto.Gen.Migration.html"},{"type":"task","title":"Examples - mix ecto.gen.migration","doc":"$ mix ecto.gen.migration add_posts_table\n    $ mix ecto.gen.migration add_posts_table -r Custom.Repo\n\nThe generated migration filename will be prefixed with the current\ntimestamp in UTC which is used for versioning and ordering.\n\nBy default, the migration will be generated to the\n\"priv/YOUR_REPO/migrations\" directory of the current application\nbut it can be configured to be any subdirectory of `priv` by\nspecifying the `:priv` key under the repository configuration.\n\nThis generator will automatically open the generated file if\nyou have `ECTO_EDITOR` set in your environment variable.","ref":"Mix.Tasks.Ecto.Gen.Migration.html#module-examples"},{"type":"task","title":"Command line options - mix ecto.gen.migration","doc":"* `-r`, `--repo` - the repo to generate migration for\n  * `--no-compile` - does not compile applications before running\n  * `--no-deps-check` - does not check dependencies before running\n  * `--migrations-path` - the path to run the migrations from, defaults to `priv/repo/migrations`","ref":"Mix.Tasks.Ecto.Gen.Migration.html#module-command-line-options"},{"type":"task","title":"Configuration - mix ecto.gen.migration","doc":"If the current app configuration specifies a custom migration module\nthe generated migration code will use that rather than the default\n`Ecto.Migration`:\n\n    config :ecto_sql, migration_module: MyApplication.CustomMigrationModule","ref":"Mix.Tasks.Ecto.Gen.Migration.html#module-configuration"},{"type":"task","title":"mix ecto.load","doc":"Loads the current environment's database structure for the\ngiven repository from a previously dumped structure file.\n\nThe repository must be set under `:ecto_repos` in the\ncurrent app configuration or given via the `-r` option.\n\nThis task needs some shell utility to be present on the machine\nrunning the task.\n\n Database   | Utility needed\n :--------- | :-------------\n PostgreSQL | psql\n MySQL      | mysql","ref":"Mix.Tasks.Ecto.Load.html"},{"type":"task","title":"Example - mix ecto.load","doc":"$ mix ecto.load","ref":"Mix.Tasks.Ecto.Load.html#module-example"},{"type":"task","title":"Command line options - mix ecto.load","doc":"* `-r`, `--repo` - the repo to load the structure info into\n  * `-d`, `--dump-path` - the path of the dump file to load from\n  * `-q`, `--quiet` - run the command quietly\n  * `-f`, `--force` - do not ask for confirmation when loading data.\n    Configuration is asked only when `:start_permanent` is set to true\n    (typically in production)\n  * `--no-compile` - does not compile applications before loading\n  * `--no-deps-check` - does not check dependencies before loading\n  * `--skip-if-loaded` - does not load the dump file if the repo has the migrations table up","ref":"Mix.Tasks.Ecto.Load.html#module-command-line-options"},{"type":"task","title":"mix ecto.migrate","doc":"Runs the pending migrations for the given repository.\n\nMigrations are expected at \"priv/YOUR_REPO/migrations\" directory\nof the current application, where \"YOUR_REPO\" is the last segment\nin your repository name. For example, the repository `MyApp.Repo`\nwill use \"priv/repo/migrations\". The repository `Whatever.MyRepo`\nwill use \"priv/my_repo/migrations\".\n\nYou can configure a repository to use another directory by specifying\nthe `:priv` key under the repository configuration. The \"migrations\"\npart will be automatically appended to it. For instance, to use\n\"priv/custom_repo/migrations\":\n\n    config :my_app, MyApp.Repo, priv: \"priv/custom_repo\"\n\nThis task runs all pending migrations by default. To migrate up to a\nspecific version number, supply `--to version_number`. To migrate a\nspecific number of times, use `--step n`.\n\nThe repositories to migrate are the ones specified under the\n`:ecto_repos` option in the current app configuration. However,\nif the `-r` option is given, it replaces the `:ecto_repos` config.\n\nSince Ecto tasks can only be executed once, if you need to migrate\nmultiple repositories, set `:ecto_repos` accordingly or pass the `-r`\nflag multiple times.\n\nIf a repository has not yet been started, one will be started outside\nyour application supervision tree and shutdown afterwards.","ref":"Mix.Tasks.Ecto.Migrate.html"},{"type":"task","title":"Examples - mix ecto.migrate","doc":"$ mix ecto.migrate\n    $ mix ecto.migrate -r Custom.Repo\n\n    $ mix ecto.migrate -n 3\n    $ mix ecto.migrate --step 3\n\n    $ mix ecto.migrate --to 20080906120000","ref":"Mix.Tasks.Ecto.Migrate.html#module-examples"},{"type":"task","title":"Command line options - mix ecto.migrate","doc":"* `--all` - run all pending migrations\n\n  * `--log-migrations-sql` - log SQL generated by migration commands\n\n  * `--log-migrator-sql` - log SQL generated by the migrator, such as\n    transactions, table locks, etc\n\n  * `--log-level` (since v3.11.0) - the level to set for `Logger`. This task\n    does not start your application, so whatever level you have configured in\n    your config files will not be used. If this is not provided, no level\n    will be set, so that if you set it yourself before calling this task\n    then this won't interfere. Can be any of the `t:Logger.level/0` levels\n\n  * `--migrations-path` - the path to load the migrations from, defaults to\n    `\"priv/repo/migrations\"`. This option may be given multiple times in which\n    case the migrations are loaded from all the given directories and sorted\n    as if they were in the same one\n\n  * `--no-compile` - does not compile applications before migrating\n\n  * `--no-deps-check` - does not check dependencies before migrating\n\n  * `--pool-size` - the pool size if the repository is started\n    only for the task (defaults to 2)\n\n  * `--prefix` - the prefix to run migrations on\n\n  * `--quiet` - do not log migration commands\n\n  * `-r`, `--repo` - the repo to migrate\n\n  * `--step`, `-n` - run n number of pending migrations\n\n  * `--strict-version-order` - abort when applying a migration with old\n    timestamp (otherwise it emits a warning)\n\n  * `--to` - run all migrations up to and including version\n\n  * `--to-exclusive` - run all migrations up to and excluding version","ref":"Mix.Tasks.Ecto.Migrate.html#module-command-line-options"},{"type":"task","title":"mix ecto.migrations","doc":"Displays the up / down migration status for the given repository.\n\nThe repository must be set under `:ecto_repos` in the\ncurrent app configuration or given via the `-r` option.\n\nBy default, migrations are expected at \"priv/YOUR_REPO/migrations\"\ndirectory of the current application but it can be configured\nby specifying the `:priv` key under the repository configuration.\n\nIf the repository has not been started yet, one will be\nstarted outside our application supervision tree and shutdown\nafterwards.","ref":"Mix.Tasks.Ecto.Migrations.html"},{"type":"task","title":"Examples - mix ecto.migrations","doc":"$ mix ecto.migrations\n    $ mix ecto.migrations -r Custom.Repo","ref":"Mix.Tasks.Ecto.Migrations.html#module-examples"},{"type":"task","title":"Command line options - mix ecto.migrations","doc":"* `--migrations-path` - the path to load the migrations from, defaults to\n    `\"priv/repo/migrations\"`. This option may be given multiple times in which\n    case the migrations are loaded from all the given directories and sorted as\n    if they were in the same one.\n\n    Note, if you have previously run migrations from paths `a/` and `b/`, and now\n    run `mix ecto.migrations --migrations-path a/` (omitting path `b/`), the\n    migrations from the path `b/` will be shown in the output as `** FILE NOT FOUND **`.\n\n  * `--no-compile` - does not compile applications before running\n\n  * `--no-deps-check` - does not check dependencies before running\n\n  * `--prefix` - the prefix to check migrations on\n\n  * `-r`, `--repo` - the repo to obtain the status for","ref":"Mix.Tasks.Ecto.Migrations.html#module-command-line-options"},{"type":"task","title":"mix ecto.rollback","doc":"Reverts applied migrations in the given repository.\n\nMigrations are expected at \"priv/YOUR_REPO/migrations\" directory\nof the current application, where \"YOUR_REPO\" is the last segment\nin your repository name. For example, the repository `MyApp.Repo`\nwill use \"priv/repo/migrations\". The repository `Whatever.MyRepo`\nwill use \"priv/my_repo/migrations\".\n\nYou can configure a repository to use another directory by specifying\nthe `:priv` key under the repository configuration. The \"migrations\"\npart will be automatically appended to it. For instance, to use\n\"priv/custom_repo/migrations\":\n\n    config :my_app, MyApp.Repo, priv: \"priv/custom_repo\"\n\nThis task rolls back the last applied migration by default. To roll\nback to a version number, supply `--to version_number`. To roll\nback a specific number of times, use `--step n`. To undo all applied\nmigrations, provide `--all`.\n\nThe repositories to rollback are the ones specified under the\n`:ecto_repos` option in the current app configuration. However,\nif the `-r` option is given, it replaces the `:ecto_repos` config.\n\nIf a repository has not yet been started, one will be started outside\nyour application supervision tree and shutdown afterwards.","ref":"Mix.Tasks.Ecto.Rollback.html"},{"type":"task","title":"Examples - mix ecto.rollback","doc":"$ mix ecto.rollback\n    $ mix ecto.rollback -r Custom.Repo\n\n    $ mix ecto.rollback -n 3\n    $ mix ecto.rollback --step 3\n\n    $ mix ecto.rollback --to 20080906120000","ref":"Mix.Tasks.Ecto.Rollback.html#module-examples"},{"type":"task","title":"Command line options - mix ecto.rollback","doc":"* `--all` - run all pending migrations\n\n  * `--log-migrations-sql` - log SQL generated by migration commands\n\n  * `--log-migrator-sql` - log SQL generated by the migrator, such as\n    transactions, table locks, etc\n\n  * `--log-level` (since v3.11.0) - the level to set for `Logger`. This task\n    does not start your application, so whatever level you have configured in\n    your config files will not be used. If this is not provided, no level\n    will be set, so that if you set it yourself before calling this task\n    then this won't interfere. Can be any of the `t:Logger.level/0` levels\n\n  * `--migrations-path` - the path to load the migrations from, defaults to\n    `\"priv/repo/migrations\"`. This option may be given multiple times in which\n    case the migrations are loaded from all the given directories and sorted\n    as if they were in the same one\n\n  * `--no-compile` - does not compile applications before migrating\n\n  * `--no-deps-check` - does not check dependencies before migrating\n\n  * `--pool-size` - the pool size if the repository is started\n    only for the task (defaults to 2)\n\n  * `--prefix` - the prefix to run migrations on\n\n  * `--quiet` - do not log migration commands\n\n  * `-r`, `--repo` - the repo to migrate\n\n  * `--step`, `-n` - revert n migrations\n\n  * `--strict-version-order` - abort when applying a migration with old\n    timestamp (otherwise it emits a warning)\n\n  * `--to` - revert all migrations down to and including version\n\n  * `--to-exclusive` - revert all migrations down to and excluding version","ref":"Mix.Tasks.Ecto.Rollback.html#module-command-line-options"},{"type":"module","title":"Ecto.Adapters.MyXQL","doc":"Adapter module for MySQL.\n\nIt uses `MyXQL` for communicating to the database.","ref":"Ecto.Adapters.MyXQL.html"},{"type":"module","title":"Options - Ecto.Adapters.MyXQL","doc":"MySQL options split in different categories described\nbelow. All options can be given via the repository\nconfiguration:","ref":"Ecto.Adapters.MyXQL.html#module-options"},{"type":"module","title":"Connection options - Ecto.Adapters.MyXQL","doc":"* `:protocol` - Set to `:socket` for using UNIX domain socket, or `:tcp` for TCP\n    (default: `:socket`)\n  * `:socket` - Connect to MySQL via UNIX sockets in the given path.\n  * `:hostname` - Server hostname\n  * `:port` - Server port (default: 3306)\n  * `:username` - Username\n  * `:password` - User password\n  * `:database` - the database to connect to\n  * `:pool` - The connection pool module, may be set to `Ecto.Adapters.SQL.Sandbox`\n  * `:ssl` - Accepts a list of options to enable TLS for the client connection,\n    or `false` to disable it. See the documentation for [Erlang's `ssl` module](`e:ssl:ssl`)\n    for a list of options (default: false)\n  * `:connect_timeout` - The timeout for establishing new connections (default: 5000)\n  * `:cli_protocol` - The protocol used for the mysql client connection (default: `\"tcp\"`).\n    This option is only used for `mix ecto.load` and `mix ecto.dump`,\n    via the `mysql` command. For more information, please check\n    [MySQL docs](https://dev.mysql.com/doc/en/connecting.html)\n  * `:socket_options` - Specifies socket configuration\n  * `:show_sensitive_data_on_connection_error` - show connection data and\n    configuration whenever there is an error attempting to connect to the\n    database\n\nThe `:socket_options` are particularly useful when configuring the size\nof both send and receive buffers. For example, when Ecto starts with a\npool of 20 connections, the memory usage may quickly grow from 20MB to\n50MB based on the operating system default values for TCP buffers. It is\nadvised to stick with the operating system defaults but they can be\ntweaked if desired:\n\n    socket_options: [recbuf: 8192, sndbuf: 8192]\n\nWe also recommend developers to consult the `MyXQL.start_link/1` documentation\nfor a complete listing of all supported options.","ref":"Ecto.Adapters.MyXQL.html#module-connection-options"},{"type":"module","title":"Storage options - Ecto.Adapters.MyXQL","doc":"* `:charset` - the database encoding (default: \"utf8mb4\")\n  * `:collation` - the collation order\n  * `:dump_path` - where to place dumped structures\n  * `:dump_prefixes` - list of prefixes that will be included in the\n    structure dump. For MySQL, this list must be of length 1. Multiple\n    prefixes are not supported. When specified, the prefixes will have\n    their definitions dumped along with the data in their migration table.\n    When it is not specified, only the configured database and its migration\n    table are dumped.","ref":"Ecto.Adapters.MyXQL.html#module-storage-options"},{"type":"module","title":"After connect callback - Ecto.Adapters.MyXQL","doc":"If you want to execute a callback as soon as connection is established\nto the database, you can use the `:after_connect` configuration. For\nexample, in your repository configuration you can add:\n\n    after_connect: {MyXQL, :query!, [\"SET variable = value\", []]}\n\nYou can also specify your own module that will receive the MyXQL\nconnection as argument.","ref":"Ecto.Adapters.MyXQL.html#module-after-connect-callback"},{"type":"module","title":"Limitations - Ecto.Adapters.MyXQL","doc":"There are some limitations when using Ecto with MySQL that one\nneeds to be aware of.","ref":"Ecto.Adapters.MyXQL.html#module-limitations"},{"type":"module","title":"Engine - Ecto.Adapters.MyXQL","doc":"Tables created by Ecto are guaranteed to use InnoDB, regardless\nof the MySQL version.","ref":"Ecto.Adapters.MyXQL.html#module-engine"},{"type":"module","title":"UUIDs - Ecto.Adapters.MyXQL","doc":"MySQL does not support UUID types. Ecto emulates them by using\n`binary(16)`.","ref":"Ecto.Adapters.MyXQL.html#module-uuids"},{"type":"module","title":"Read after writes - Ecto.Adapters.MyXQL","doc":"Because MySQL does not support RETURNING clauses in INSERT and\nUPDATE, it does not support the `:read_after_writes` option of\n`Ecto.Schema.field/3`.","ref":"Ecto.Adapters.MyXQL.html#module-read-after-writes"},{"type":"module","title":"DDL Transaction - Ecto.Adapters.MyXQL","doc":"MySQL does not support migrations inside transactions as it\nautomatically commits after some commands like CREATE TABLE.\nTherefore MySQL migrations does not run inside transactions.","ref":"Ecto.Adapters.MyXQL.html#module-ddl-transaction"},{"type":"module","title":"Old MySQL versions - Ecto.Adapters.MyXQL","doc":"","ref":"Ecto.Adapters.MyXQL.html#module-old-mysql-versions"},{"type":"module","title":"JSON support - Ecto.Adapters.MyXQL","doc":"MySQL introduced a native JSON type in v5.7.8, if your server is\nusing this version or higher, you may use `:map` type for your\ncolumn in migration:\n\n    add :some_field, :map\n\nIf you're using older server versions, use a `TEXT` field instead:\n\n    add :some_field, :text\n\nin either case, the adapter will automatically encode/decode the\nvalue from JSON.","ref":"Ecto.Adapters.MyXQL.html#module-json-support"},{"type":"module","title":"usec in datetime - Ecto.Adapters.MyXQL","doc":"Old MySQL versions did not support usec in datetime while\nmore recent versions would round or truncate the usec value.\n\nTherefore, in case the user decides to use microseconds in\ndatetimes and timestamps with MySQL, be aware of such\ndifferences and consult the documentation for your MySQL\nversion.\n\nIf your version of MySQL supports microsecond precision, you\nwill be able to utilize Ecto's usec types.","ref":"Ecto.Adapters.MyXQL.html#module-usec-in-datetime"},{"type":"module","title":"Multiple Result Support - Ecto.Adapters.MyXQL","doc":"MyXQL supports the execution of queries that return multiple\nresults, such as text queries with multiple statements separated\nby semicolons or stored procedures. These can be executed with\n`Ecto.Adapters.SQL.query_many/4` or the `YourRepo.query_many/3`\nshortcut.\n\nBe default, these queries will be executed with the `:query_type`\noption set to `:text`. To take advantage of prepared statements\nwhen executing a stored procedure, set the `:query_type` option\nto `:binary`.","ref":"Ecto.Adapters.MyXQL.html#module-multiple-result-support"},{"type":"module","title":"Ecto.Adapters.Postgres","doc":"Adapter module for PostgreSQL.\n\nIt uses `Postgrex` for communicating to the database.","ref":"Ecto.Adapters.Postgres.html"},{"type":"module","title":"Features - Ecto.Adapters.Postgres","doc":"* Full query support (including joins, preloads and associations)\n  * Support for transactions\n  * Support for data migrations\n  * Support for ecto.create and ecto.drop operations\n  * Support for transactional tests via `Ecto.Adapters.SQL`","ref":"Ecto.Adapters.Postgres.html#module-features"},{"type":"module","title":"Options - Ecto.Adapters.Postgres","doc":"Postgres options split in different categories described\nbelow. All options can be given via the repository\nconfiguration:\n\n    config :your_app, YourApp.Repo,\n      ...\n\nThe `:prepare` option may be specified per operation:\n\n    YourApp.Repo.all(Queryable, prepare: :unnamed)","ref":"Ecto.Adapters.Postgres.html#module-options"},{"type":"module","title":"Migration options - Ecto.Adapters.Postgres","doc":"* `:migration_lock` - prevent multiple nodes from running migrations at the same\n    time by obtaining a lock. The value `:table_lock` will lock migrations by wrapping\n    the entire migration inside a database transaction, including inserting the\n    migration version into the migration source (by default, \"schema_migrations\").\n    You may alternatively select `:pg_advisory_lock` which has the benefit\n    of allowing concurrent operations such as creating indexes. (default: `:table_lock`)\n\nWhen using the `:pg_advisory_lock` migration lock strategy and Ecto cannot obtain\nthe lock due to another instance occupying the lock, Ecto will wait for 5 seconds\nand then retry infinity times. This is configurable on the repo with keys\n`:migration_advisory_lock_retry_interval_ms` and `:migration_advisory_lock_max_tries`.\nIf the retries are exhausted, the migration will fail.\n\nSome downsides to using advisory locks is that some Postgres-compatible systems or plugins\nmay not support session level locks well and therefore result in inconsistent behavior.\nFor example, PgBouncer when using pool_modes other than session won't work well with\nadvisory locks. CockroachDB is another system that is designed in a way that advisory\nlocks don't make sense for their distributed database.","ref":"Ecto.Adapters.Postgres.html#module-migration-options"},{"type":"module","title":"Connection options - Ecto.Adapters.Postgres","doc":"* `:hostname` - Server hostname\n  * `:socket_dir` - Connect to Postgres via UNIX sockets in the given directory\n    The socket name is derived based on the port. This is the preferred method\n    for configuring sockets and it takes precedence over the hostname. If you are\n    connecting to a socket outside of the Postgres convention, use `:socket` instead;\n  * `:socket` - Connect to Postgres via UNIX sockets in the given path.\n    This option takes precedence over the `:hostname` and `:socket_dir`\n  * `:username` - Username\n  * `:password` - User password\n  * `:port` - Server port (default: 5432)\n  * `:database` - the database to connect to\n  * `:maintenance_database` - Specifies the name of the database to connect to when\n    creating or dropping the database. Defaults to `\"postgres\"`\n  * `:pool` - The connection pool module, may be set to `Ecto.Adapters.SQL.Sandbox`\n  * `:ssl` - Accepts a list of options to enable TLS for the client connection,\n    or `false` to disable it. See the documentation for [Erlang's `ssl` module](`e:ssl:ssl`)\n    for a list of options (default: false)\n  * `:parameters` - Keyword list of connection parameters\n  * `:connect_timeout` - The timeout for establishing new connections (default: 5000)\n  * `:prepare` - How to prepare queries, either `:named` to use named queries\n    or `:unnamed` to force unnamed queries (default: `:named`)\n  * `:socket_options` - Specifies socket configuration\n  * `:show_sensitive_data_on_connection_error` - show connection data and\n    configuration whenever there is an error attempting to connect to the\n    database\n\nThe `:socket_options` are particularly useful when configuring the size\nof both send and receive buffers. For example, when Ecto starts with a\npool of 20 connections, the memory usage may quickly grow from 20MB to\n50MB based on the operating system default values for TCP buffers. It is\nadvised to stick with the operating system defaults but they can be\ntweaked if desired:\n\n    socket_options: [recbuf: 8192, sndbuf: 8192]\n\nWe also recommend developers to consult the `Postgrex.start_link/1`\ndocumentation for a complete listing of all supported options.","ref":"Ecto.Adapters.Postgres.html#module-connection-options"},{"type":"module","title":"Storage options - Ecto.Adapters.Postgres","doc":"* `:encoding` - the database encoding (default: \"UTF8\")\n    or `:unspecified` to remove encoding parameter (alternative engine compatibility)\n  * `:template` - the template to create the database from\n  * `:lc_collate` - the collation order\n  * `:lc_ctype` - the character classification\n  * `:dump_path` - where to place dumped structures\n  * `dump_prefixes` - list of prefixes that will be included in the structure dump.\n    When specified, the prefixes will have their definitions dumped along with the\n    data in their migration table. When it is not specified, the configured\n    database has the definitions dumped from all of its schemas but only\n    the data from the migration table from the `public` schema is included.\n  * `:force_drop` - force the database to be dropped even\n    if it has connections to it (requires PostgreSQL 13+)","ref":"Ecto.Adapters.Postgres.html#module-storage-options"},{"type":"module","title":"After connect callback - Ecto.Adapters.Postgres","doc":"If you want to execute a callback as soon as connection is established\nto the database, you can use the `:after_connect` configuration. For\nexample, in your repository configuration you can add:\n\n    after_connect: {Postgrex, :query!, [\"SET search_path TO global_prefix\", []]}\n\nYou can also specify your own module that will receive the Postgrex\nconnection as argument.","ref":"Ecto.Adapters.Postgres.html#module-after-connect-callback"},{"type":"module","title":"Extensions - Ecto.Adapters.Postgres","doc":"Both PostgreSQL and its adapter for Elixir, Postgrex, support an\nextension system. If you want to use custom extensions for Postgrex\nalongside Ecto, you must define a type module with your extensions.\nCreate a new file anywhere in your application with the following:\n\n    Postgrex.Types.define(MyApp.PostgresTypes,\n                          [MyExtension.Foo, MyExtensionBar] ++ Ecto.Adapters.Postgres.extensions())\n\nOnce your type module is defined, you can configure the repository to use it:\n\n    config :my_app, MyApp.Repo, types: MyApp.PostgresTypes","ref":"Ecto.Adapters.Postgres.html#module-extensions"},{"type":"function","title":"Ecto.Adapters.Postgres.extensions/0","doc":"All Ecto extensions for Postgrex.","ref":"Ecto.Adapters.Postgres.html#extensions/0"},{"type":"module","title":"Ecto.Adapters.Tds","doc":"Adapter module for MSSQL Server using the TDS protocol.","ref":"Ecto.Adapters.Tds.html"},{"type":"module","title":"Options - Ecto.Adapters.Tds","doc":"Tds options split in different categories described\nbelow. All options can be given via the repository\nconfiguration.","ref":"Ecto.Adapters.Tds.html#module-options"},{"type":"module","title":"Connection options - Ecto.Adapters.Tds","doc":"* `:hostname` - Server hostname\n  * `:port` - Server port (default: 1433)\n  * `:username` - Username\n  * `:password` - User password\n  * `:database` - the database to connect to\n  * `:pool` - The connection pool module, may be set to `Ecto.Adapters.SQL.Sandbox`\n  * `:ssl` - Set to true if ssl should be used (default: false)\n  * `:ssl_opts` - A list of ssl options, see Erlang's `ssl` docs\n  * `:show_sensitive_data_on_connection_error` - show connection data and\n    configuration whenever there is an error attempting to connect to the\n    database\n\nWe also recommend developers to consult the `Tds.start_link/1` documentation\nfor a complete list of all supported options for driver.","ref":"Ecto.Adapters.Tds.html#module-connection-options"},{"type":"module","title":"Storage options - Ecto.Adapters.Tds","doc":"* `:collation` - the database collation. Used during database creation but\n    it is ignored later\n\nIf you need collation other than Latin1, add `tds_encoding` as dependency to\nyour project `mix.exs` file then amend `config/config.ex` by adding:\n\n    config :tds, :text_encoder, Tds.Encoding\n\nThis should give you extended set of most encoding. For complete list check\n`Tds.Encoding` [documentation](https://hexdocs.pm/tds_encoding).","ref":"Ecto.Adapters.Tds.html#module-storage-options"},{"type":"module","title":"After connect flags - Ecto.Adapters.Tds","doc":"After connecting to MSSQL server, TDS will check if there are any flags set in\nconnection options that should affect connection session behaviour. All flags are\nMSSQL standard *SET* options. The following flags are currently supported:\n\n  * `:set_language` - sets session language (consult stored procedure output\n     `exec sp_helplanguage` for valid values)\n  * `:set_datefirst` - number in range 1..7\n  * `:set_dateformat` - atom, one of `:mdy | :dmy | :ymd | :ydm | :myd | :dym`\n  * `:set_deadlock_priority` - atom, one of `:low | :high | :normal | -10..10`\n  * `:set_lock_timeout` - number in milliseconds > 0\n  * `:set_remote_proc_transactions` - atom, one of `:on | :off`\n  * `:set_implicit_transactions` - atom, one of `:on | :off`\n  * `:set_allow_snapshot_isolation` - atom, one of `:on | :off`\n     (required if `Repo.transaction(fn -> ... end, isolation_level: :snapshot)` is used)\n  * `:set_read_committed_snapshot` - atom, one of `:on | :off`","ref":"Ecto.Adapters.Tds.html#module-after-connect-flags"},{"type":"module","title":"Limitations - Ecto.Adapters.Tds","doc":"","ref":"Ecto.Adapters.Tds.html#module-limitations"},{"type":"module","title":"UUIDs - Ecto.Adapters.Tds","doc":"MSSQL server has slightly different binary storage format for UUIDs (`uniqueidentifier`).\nIf you use `:binary_id`, the proper choice is made. Otherwise you must use the `Tds.Ecto.UUID`\ntype. Avoid using `Ecto.UUID` since it may cause unpredictable application behaviour.","ref":"Ecto.Adapters.Tds.html#module-uuids"},{"type":"module","title":"SQL `Char`, `VarChar` and `Text` types - Ecto.Adapters.Tds","doc":"When working with binaries and strings,there are some limitations you should be aware of:\n\n  - Strings that should be stored in mentioned sql types must be encoded to column\n    codepage (defined in collation). If collation is different than database collation,\n    it is not possible to store correct value into database since the connection\n    respects the database collation. Ecto does not provide way to override parameter\n    codepage.\n\n  - If you need other than Latin1 or other than your database default collation, as\n    mentioned in \"Storage Options\" section, then manually encode strings using\n    `Tds.Encoding.encode/2` into desired codepage and then tag parameter as `:binary`.\n    Please be aware that queries that use this approach in where clauses can be 10x slower\n    due increased logical reads in database.\n\n  - You can't store VarChar codepoints encoded in one collation/codepage to column that\n    is encoded in different collation/codepage. You will always get wrong result. This is\n    not adapter or driver limitation but rather how string encoding works for single byte\n    encoded strings in MSSQL server. Don't be confused if you are always seeing latin1 chars,\n    they are simply in each codepoint table.\n\nIn particular, if a field has the type `:text`, only raw binaries will be allowed.\nTo avoid above limitations always use `:string` (NVarChar) type for text if possible.\nIf you really need to use VarChar's column type, you can use the `Tds.Ecto.VarChar`\nEcto type.","ref":"Ecto.Adapters.Tds.html#module-sql-char-varchar-and-text-types"},{"type":"module","title":"JSON support - Ecto.Adapters.Tds","doc":"Even though the adapter will convert `:map` fields into JSON back and forth,\nactual value is stored in NVarChar column.","ref":"Ecto.Adapters.Tds.html#module-json-support"},{"type":"module","title":"Query hints and table hints - Ecto.Adapters.Tds","doc":"MSSQL supports both query hints and table hints: https://docs.microsoft.com/en-us/sql/t-sql/queries/hints-transact-sql-query\n\nFor Ecto compatibility, the query hints must be given via the `lock` option, and they\nwill be translated to MSSQL's \"OPTION\". If you need to pass multiple options, you\ncan separate them by comma:\n\n    from query, lock: \"HASH GROUP, FAST 10\"\n\nTable hints are specified as a list alongside a `from` or `join`:\n\n    from query, hints: [\"INDEX (IX_Employee_ManagerID)\"]\n\nThe `:migration_lock` will be treated as a table hint and defaults to \"UPDLOCK\".","ref":"Ecto.Adapters.Tds.html#module-query-hints-and-table-hints"},{"type":"module","title":"Multi Repo calls in transactions - Ecto.Adapters.Tds","doc":"To avoid deadlocks in your app, we exposed `:isolation_level`  repo transaction option.\nThis will tell to SQL Server Transaction Manager how to begin transaction.\nBy default, if this option is omitted, isolation level is set to `:read_committed`.\n\nAny attempt to manually set the transaction isolation via queries, such as\n\n    Ecto.Adapter.SQL.query(\"SET TRANSACTION ISOLATION LEVEL XYZ\")\n\nwill fail once explicit transaction is started using `c:Ecto.Repo.transaction/2`\nand reset back to :read_committed.\n\nThere is `Ecto.Query.lock/3` function can help by setting it to `WITH(NOLOCK)`.\nThis should allow you to do eventually consistent reads and avoid locks on given\ntable if you don't need to write to database.\n\nNOTE: after explicit transaction ends (commit or rollback) implicit transactions\nwill run as READ_COMMITTED.","ref":"Ecto.Adapters.Tds.html#module-multi-repo-calls-in-transactions"},{"type":"module","title":"Tds.Ecto.UUID","doc":"A TDS adapter type for UUIDs strings.\n\nIf you are using Tds adapter and UUIDs in your project, instead of `Ecto.UUID`\nyou should use Tds.Ecto.UUID to generate correct bytes that should be stored\nin database.","ref":"Tds.Ecto.UUID.html"},{"type":"function","title":"Tds.Ecto.UUID.bingenerate/0","doc":"Generates a version 4 (random) UUID in the binary format.","ref":"Tds.Ecto.UUID.html#bingenerate/0"},{"type":"function","title":"Tds.Ecto.UUID.cast/1","doc":"Casts to UUID.","ref":"Tds.Ecto.UUID.html#cast/1"},{"type":"function","title":"Tds.Ecto.UUID.cast!/1","doc":"Same as `cast/1` but raises `Ecto.CastError` on invalid arguments.","ref":"Tds.Ecto.UUID.html#cast!/1"},{"type":"function","title":"Tds.Ecto.UUID.dump/1","doc":"Converts a string representing a UUID into a binary.","ref":"Tds.Ecto.UUID.html#dump/1"},{"type":"function","title":"Tds.Ecto.UUID.dump!/1","doc":"","ref":"Tds.Ecto.UUID.html#dump!/1"},{"type":"function","title":"Tds.Ecto.UUID.embed_as/1","doc":"","ref":"Tds.Ecto.UUID.html#embed_as/1"},{"type":"function","title":"Tds.Ecto.UUID.equal?/2","doc":"","ref":"Tds.Ecto.UUID.html#equal?/2"},{"type":"function","title":"Tds.Ecto.UUID.generate/0","doc":"Generates a version 4 (random) UUID.","ref":"Tds.Ecto.UUID.html#generate/0"},{"type":"function","title":"Tds.Ecto.UUID.load/1","doc":"Converts a binary UUID into a string.","ref":"Tds.Ecto.UUID.html#load/1"},{"type":"type","title":"Tds.Ecto.UUID.raw/0","doc":"A raw binary representation of a UUID.","ref":"Tds.Ecto.UUID.html#t:raw/0"},{"type":"type","title":"Tds.Ecto.UUID.t/0","doc":"A hex-encoded UUID string.","ref":"Tds.Ecto.UUID.html#t:t/0"},{"type":"module","title":"Tds.Ecto.VarChar","doc":"A Tds adapter Ecto Type that wraps erlang string into tuple so TDS driver\ncan understand if erlang string should be encoded as NVarChar or Varchar.\n\nDue to some limitations in Ecto and Tds driver, it is not possible to\nsupport collations other than the one set on connection during login.\nPlease be aware of this limitation if you plan to store varchar values in\nyour database using Ecto since you will probably lose some codepoints in\nthe value during encoding. Instead use `tds_encoding` library and first\nencode value and then annotate it as `:binary` by calling `Ecto.Query.API.type/2`\nin your query. This way all codepoints will be properly preserved during\ninsert to database.","ref":"Tds.Ecto.VarChar.html"},{"type":"function","title":"Tds.Ecto.VarChar.cast/1","doc":"Casts to string.","ref":"Tds.Ecto.VarChar.html#cast/1"},{"type":"function","title":"Tds.Ecto.VarChar.cast!/1","doc":"Same as `cast/1` but raises `Ecto.CastError` on invalid arguments.","ref":"Tds.Ecto.VarChar.html#cast!/1"},{"type":"function","title":"Tds.Ecto.VarChar.dump/1","doc":"Converts a string representing a VarChar into a tuple `{value, :varchar}`.\n\nReturns `:error` if value is not binary.","ref":"Tds.Ecto.VarChar.html#dump/1"},{"type":"function","title":"Tds.Ecto.VarChar.embed_as/1","doc":"","ref":"Tds.Ecto.VarChar.html#embed_as/1"},{"type":"function","title":"Tds.Ecto.VarChar.equal?/2","doc":"","ref":"Tds.Ecto.VarChar.html#equal?/2"},{"type":"function","title":"Tds.Ecto.VarChar.load/1","doc":"Loads the DB type as is.","ref":"Tds.Ecto.VarChar.html#load/1"},{"type":"type","title":"Tds.Ecto.VarChar.t/0","doc":"An erlang string","ref":"Tds.Ecto.VarChar.html#t:t/0"},{"type":"type","title":"Tds.Ecto.VarChar.varchar/0","doc":"A value annotated as varchar.","ref":"Tds.Ecto.VarChar.html#t:varchar/0"},{"type":"behaviour","title":"Ecto.Adapter.Migration","doc":"Specifies the adapter migrations API.","ref":"Ecto.Adapter.Migration.html"},{"type":"callback","title":"Ecto.Adapter.Migration.execute_ddl/3","doc":"Executes migration commands.","ref":"Ecto.Adapter.Migration.html#c:execute_ddl/3"},{"type":"callback","title":"Ecto.Adapter.Migration.lock_for_migrations/3","doc":"Locks the migrations table and emits the locked versions for callback execution.\n\nIt returns the result of calling the given function with a list of versions.","ref":"Ecto.Adapter.Migration.html#c:lock_for_migrations/3"},{"type":"callback","title":"Ecto.Adapter.Migration.supports_ddl_transaction?/0","doc":"Checks if the adapter supports ddl transaction.","ref":"Ecto.Adapter.Migration.html#c:supports_ddl_transaction?/0"},{"type":"type","title":"Ecto.Adapter.Migration.adapter_meta/0","doc":"","ref":"Ecto.Adapter.Migration.html#t:adapter_meta/0"},{"type":"type","title":"Ecto.Adapter.Migration.command/0","doc":"All migration commands","ref":"Ecto.Adapter.Migration.html#t:command/0"},{"type":"type","title":"Ecto.Adapter.Migration.ddl_object/0","doc":"A struct that represents a table or index in a database schema.\n\nThese database objects can be modified through the use of a Data\nDefinition Language, hence the name DDL object.","ref":"Ecto.Adapter.Migration.html#t:ddl_object/0"},{"type":"type","title":"Ecto.Adapter.Migration.drop_mode/0","doc":"","ref":"Ecto.Adapter.Migration.html#t:drop_mode/0"},{"type":"type","title":"Ecto.Adapter.Migration.table_subcommand/0","doc":"All commands allowed within the block passed to `table/2`","ref":"Ecto.Adapter.Migration.html#t:table_subcommand/0"},{"type":"behaviour","title":"Ecto.Adapter.Structure","doc":"Specifies the adapter structure (dump/load) API.","ref":"Ecto.Adapter.Structure.html"},{"type":"callback","title":"Ecto.Adapter.Structure.dump_cmd/3","doc":"Runs the dump command for the given repo / config.\n\nCalling this function will setup authentication and run the dump cli\ncommand with your provided `args`.\n\nThe options in `opts` are passed directly to `System.cmd/3`.\n\nReturns `{output, exit_status}` where `output` is a string of the stdout\n(as long as no option `into` is provided, see `System.cmd/3`) and `exit_status`\nis the exit status of the invocation. (`0` for success)","ref":"Ecto.Adapter.Structure.html#c:dump_cmd/3"},{"type":"callback","title":"Examples - Ecto.Adapter.Structure.dump_cmd/3","doc":"iex> dump_cmd([\"--data-only\", \"--table\", \"table_name\"], [stdout_to_stderr: true], Acme.Repo.config())\n    {\"--\n-- PostgreSQL database dump\n--\n\" <> _rest, 0}","ref":"Ecto.Adapter.Structure.html#c:dump_cmd/3-examples"},{"type":"callback","title":"Ecto.Adapter.Structure.structure_dump/2","doc":"Dumps the given structure.\n\nThe path will be looked in the `config` under :dump_path or\ndefault to the structure path inside `default`.\n\nReturns an `:ok` tuple if it was dumped successfully, an error tuple otherwise.","ref":"Ecto.Adapter.Structure.html#c:structure_dump/2"},{"type":"callback","title":"Examples - Ecto.Adapter.Structure.structure_dump/2","doc":"structure_dump(\"priv/repo\", username: \"postgres\",\n                                database: \"ecto_test\",\n                                hostname: \"localhost\")","ref":"Ecto.Adapter.Structure.html#c:structure_dump/2-examples"},{"type":"callback","title":"Ecto.Adapter.Structure.structure_load/2","doc":"Loads the given structure.\n\nThe path will be looked in the `config` under :dump_path or\ndefault to the structure path inside `default`.\n\nReturns an `:ok` tuple if it was loaded successfully, an error tuple otherwise.","ref":"Ecto.Adapter.Structure.html#c:structure_load/2"},{"type":"callback","title":"Examples - Ecto.Adapter.Structure.structure_load/2","doc":"structure_load(\"priv/repo\", username: \"postgres\",\n                                database: \"ecto_test\",\n                                hostname: \"localhost\")","ref":"Ecto.Adapter.Structure.html#c:structure_load/2-examples"},{"type":"behaviour","title":"Ecto.Adapters.SQL.Connection","doc":"Specifies the behaviour to be implemented by all SQL connections.","ref":"Ecto.Adapters.SQL.Connection.html"},{"type":"callback","title":"Ecto.Adapters.SQL.Connection.all/1","doc":"Receives a query and must return a SELECT query.","ref":"Ecto.Adapters.SQL.Connection.html#c:all/1"},{"type":"callback","title":"Ecto.Adapters.SQL.Connection.child_spec/1","doc":"Receives options and returns `DBConnection` supervisor child\nspecification.","ref":"Ecto.Adapters.SQL.Connection.html#c:child_spec/1"},{"type":"callback","title":"Ecto.Adapters.SQL.Connection.ddl_logs/1","doc":"Receives a query result and returns a list of logs.","ref":"Ecto.Adapters.SQL.Connection.html#c:ddl_logs/1"},{"type":"callback","title":"Ecto.Adapters.SQL.Connection.delete/4","doc":"Returns a DELETE for the `filters` returning the given `returning`.","ref":"Ecto.Adapters.SQL.Connection.html#c:delete/4"},{"type":"callback","title":"Ecto.Adapters.SQL.Connection.delete_all/1","doc":"Receives a query and must return a DELETE query.","ref":"Ecto.Adapters.SQL.Connection.html#c:delete_all/1"},{"type":"callback","title":"Ecto.Adapters.SQL.Connection.execute/4","doc":"Executes a cached query.","ref":"Ecto.Adapters.SQL.Connection.html#c:execute/4"},{"type":"callback","title":"Ecto.Adapters.SQL.Connection.execute_ddl/1","doc":"Receives a DDL command and returns a query that executes it.","ref":"Ecto.Adapters.SQL.Connection.html#c:execute_ddl/1"},{"type":"callback","title":"Ecto.Adapters.SQL.Connection.explain_query/4","doc":"Executes an EXPLAIN query or similar depending on the adapter to obtains statistics of the given query.\n\nReceives the `connection`, `query`, `params` for the query,\nand all `opts` including those related to the EXPLAIN statement and shared opts.\n\nMust execute the explain query and return the result.","ref":"Ecto.Adapters.SQL.Connection.html#c:explain_query/4"},{"type":"callback","title":"Ecto.Adapters.SQL.Connection.insert/7","doc":"Returns an INSERT for the given `rows` in `table` returning\nthe given `returning`.","ref":"Ecto.Adapters.SQL.Connection.html#c:insert/7"},{"type":"callback","title":"Ecto.Adapters.SQL.Connection.prepare_execute/5","doc":"Prepares and executes the given query with `DBConnection`.","ref":"Ecto.Adapters.SQL.Connection.html#c:prepare_execute/5"},{"type":"callback","title":"Ecto.Adapters.SQL.Connection.query/4","doc":"Runs the given statement as a query.","ref":"Ecto.Adapters.SQL.Connection.html#c:query/4"},{"type":"callback","title":"Ecto.Adapters.SQL.Connection.query_many/4","doc":"Runs the given statement as a multi-result query.","ref":"Ecto.Adapters.SQL.Connection.html#c:query_many/4"},{"type":"callback","title":"Ecto.Adapters.SQL.Connection.stream/4","doc":"Returns a stream that prepares and executes the given query with\n`DBConnection`.","ref":"Ecto.Adapters.SQL.Connection.html#c:stream/4"},{"type":"callback","title":"Ecto.Adapters.SQL.Connection.table_exists_query/1","doc":"Returns a queryable to check if the given `table` exists.","ref":"Ecto.Adapters.SQL.Connection.html#c:table_exists_query/1"},{"type":"callback","title":"Ecto.Adapters.SQL.Connection.to_constraints/2","doc":"Receives the exception returned by `c:query/4`.\n\nThe constraints are in the keyword list and must return the\nconstraint type, like `:unique`, and the constraint name as\na string, for example:\n\n    [unique: \"posts_title_index\"]\n\nMust return an empty list if the error does not come\nfrom any constraint.","ref":"Ecto.Adapters.SQL.Connection.html#c:to_constraints/2"},{"type":"callback","title":"Ecto.Adapters.SQL.Connection.update/5","doc":"Returns an UPDATE for the given `fields` in `table` filtered by\n`filters` returning the given `returning`.","ref":"Ecto.Adapters.SQL.Connection.html#c:update/5"},{"type":"callback","title":"Ecto.Adapters.SQL.Connection.update_all/1","doc":"Receives a query and values to update and must return an UPDATE query.","ref":"Ecto.Adapters.SQL.Connection.html#c:update_all/1"},{"type":"type","title":"Ecto.Adapters.SQL.Connection.cached/0","doc":"The cached query which is a DBConnection Query","ref":"Ecto.Adapters.SQL.Connection.html#t:cached/0"},{"type":"type","title":"Ecto.Adapters.SQL.Connection.connection/0","doc":"","ref":"Ecto.Adapters.SQL.Connection.html#t:connection/0"},{"type":"type","title":"Ecto.Adapters.SQL.Connection.name/0","doc":"The query name","ref":"Ecto.Adapters.SQL.Connection.html#t:name/0"},{"type":"type","title":"Ecto.Adapters.SQL.Connection.params/0","doc":"","ref":"Ecto.Adapters.SQL.Connection.html#t:params/0"},{"type":"type","title":"Ecto.Adapters.SQL.Connection.statement/0","doc":"The SQL statement","ref":"Ecto.Adapters.SQL.Connection.html#t:statement/0"},{"type":"module","title":"Ecto.Migration.Command","doc":"Used internally by adapters.\n\nThis represents the up and down legs of a reversible raw command\nthat is usually defined with `Ecto.Migration.execute/1`.\n\nTo define a reversible command in a migration, see `Ecto.Migration.execute/2`.","ref":"Ecto.Migration.Command.html"},{"type":"type","title":"Ecto.Migration.Command.t/0","doc":"","ref":"Ecto.Migration.Command.html#t:t/0"},{"type":"module","title":"Ecto.Migration.Constraint","doc":"Used internally by adapters.\n\nTo define a constraint in a migration, see `Ecto.Migration.constraint/3`.","ref":"Ecto.Migration.Constraint.html"},{"type":"type","title":"Ecto.Migration.Constraint.t/0","doc":"","ref":"Ecto.Migration.Constraint.html#t:t/0"},{"type":"module","title":"Ecto.Migration.Index","doc":"Used internally by adapters.\n\nTo define an index in a migration, see `Ecto.Migration.index/3`.","ref":"Ecto.Migration.Index.html"},{"type":"type","title":"Ecto.Migration.Index.t/0","doc":"","ref":"Ecto.Migration.Index.html#t:t/0"},{"type":"module","title":"Ecto.Migration.Reference","doc":"Used internally by adapters.\n\nTo define a reference in a migration, see `Ecto.Migration.references/2`.","ref":"Ecto.Migration.Reference.html"},{"type":"type","title":"Ecto.Migration.Reference.t/0","doc":"The reference struct.\n\nThe `:prefix` field is deprecated and should instead be stored in the `:options` field.","ref":"Ecto.Migration.Reference.html#t:t/0"},{"type":"module","title":"Ecto.Migration.Table","doc":"Used internally by adapters.\n\nTo define a table in a migration, see `Ecto.Migration.table/2`.","ref":"Ecto.Migration.Table.html"},{"type":"type","title":"Ecto.Migration.Table.t/0","doc":"","ref":"Ecto.Migration.Table.html#t:t/0"},{"type":"extras","title":"Changelog for v3.x","doc":"# Changelog for v3.x","ref":"changelog.html"},{"type":"extras","title":"v3.12.0 (2024-08-12) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-12-0-2024-08-12"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Migration] Add `Ecto.Migration.remove_if_exists/1`\n  * [Ecto.Migrator] Warn for migration files that end in `.ex`\n  * [sql] Support for subqueries in order_bys and group_bys\n  * [mysql] Add check constraints for MySQL\n  * [postgres] Add native bitstring support to Postgres\n  * [postgres] Add support for `:duration` type\n  * [postgres] Add `:plan` explain option for Postgres\n  * [tds] Allow passing `%Tds.Parameter` structs as params for named parameter usage in `query`","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fix - Changelog for v3.x","doc":"* [mysql] Type cast of integers in MySQL should use signed integers","ref":"changelog.html#bug-fix"},{"type":"extras","title":"v3.11.3 (2024-06-13) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-11-3-2024-06-13"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [mysql] Relax `myxql` dependency","ref":"changelog.html#enhancements"},{"type":"extras","title":"v3.11.2 (2024-05-18) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-11-2-2024-05-18"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [postgres] Relax `postgrex` dependency","ref":"changelog.html#enhancements"},{"type":"extras","title":"v3.11.1 (2023-12-07) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-11-1-2023-12-07"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Migration] Add `:generated` option to columns\n  * [Ecto.Migration] Add index storage parameters (via :options) for Postgres","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [Ecto.Migration] Support `:prefix` on index rename\n  * [Ecto.Migrator] Stop runner if migration fails","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.11.0 (2023-11-14) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-11-0-2023-11-14"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [mix ecto.migrate] Add `--log-level` to ecto.migrate\n  * [mix ecto.rollback] Add `--log-level` to ecto.rollback\n  * [sql] Support fragment splicing\n  * [sql] Support data-modifying CTEs\n  * [sql] Add source to insert_ll, insert, update, and delete telemetry events\n  * [tds] Include `exec` before stored procedure for TDS (for earlier SQLServer versions)","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [mix ecto.migrate] Read existing dynamic repo in migrations\n  * [mix ecto.migrate] Don't add primary key on remove migration","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.10.2 (2023-08-21) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-10-2-2023-08-21"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [migrations] Handle `from: {reference, opts}` in FK migrations\n  * [mysql] Support MariaDB versioned tables","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [migrations] Don't add comment to removed columns\n  * [migrations] Ensure module is loaded before checking for migration\n  * [mysql] Fix for casting boolean values in MySQL","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.10.1 (2023-04-11) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-10-1-2023-04-11"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [postgres] Allow Postgrex v0.17.x","ref":"changelog.html#enhancements"},{"type":"extras","title":"v3.10.0 (2023-04-10) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-10-0-2023-04-10"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [Ecto.Migrator] Allow running the migrator in your supervision tree\n  * [Ecto.Migrator] Allow renaming an index\n  * [Ecto.Migrator] Add `execute_file/1` and `execute_file/2`\n  * [mix ecto.dump] Support dumping  multiple prefixes on PostgreSQL and MySQL\n  * [mysql] Improve constraint matching support on alternative implementations\n  * [postgres] Allow `CASCADE` when dropping a constraint on postgres","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [mix ecto.load] Suppress query logs in mix ecto.load when quiet flag is given","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.9.2 (2022-12-20) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-9-2-2022-12-20"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [migrator] Raise if target version in `to`/`exclusive_to` is not an integer\n  * [mysql] Add support for cross lateral joins\n  * [postgres] Add support for cross lateral joins\n  * [postgres] Add support for materialized CTEs\n  * [telemetry] Send `cast_params` metadata to telemetry events","ref":"changelog.html#enhancements"},{"type":"extras","title":"v3.9.1 (2022-11-18) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-9-1-2022-11-18"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [mysql] Support `:format` option on `explain`\n  * [postgres] Permit outer joins when using `update_all`\n  * [sql] Add support for `ONLY` in index creation","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [mysql] Ensure locks are quoted\n  * [mysql] Do not crash on `mix ecto.drop` when the database is unreachable\n  * [postgres] Fix empty array compare in PostgreSQL\n  * [sql] Allow function sources whose name begins with 'select'","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.9.0 (2022-09-27) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-9-0-2022-09-27"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [migrations] Support `primary_key` configuration options in `table`\n  * [migrations] Add `:nulls_distinct` option for unique indexes\n  * [postgres] Support the use of advisory locks for migrations\n  * [sql] Add `dump_cmd` to `postgrex` and `myxql` adapters\n  * [sql] Log human-readable UUIDs by using pre-dumped query parameters\n  * [sql] Support select aliases from `selected_as/1` and `selected_as/2`\n  * [telemetry] Emit `schema_migration: true` under `telemetry_options`","ref":"changelog.html#enhancements"},{"type":"extras","title":"v3.8.3 (2022-06-04) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-8-3-2022-06-04"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [sql] Implement `literal/1` support in fragments","ref":"changelog.html#enhancements"},{"type":"extras","title":"v3.8.2 (2022-05-18) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-8-2-2022-05-18"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [postgres] Fix possible breaking change on `json_extract_path` for boolean values introduced in v3.8.0\n  * [sql] Colorize stacktrace and use `:` before printing line number","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.8.1 (2022-04-29) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-8-1-2022-04-29"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [mysql] Raise on a subquery with parameter on MySQL join\n  * [sql] Do not invoke dynamic repositories in direct `Ecto.Adapters.SQL` operations","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.8.0 (2022-04-26) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-8-0-2022-04-26"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [migrations] Support `--to-exclusive` in `mix ecto.migrate` and `mix ecto.rollback`\n  * [mysql] Add `:comment` support on MySQL migrations\n  * [postgres] Support `:prepare` option per operation\n  * [postgres] Optimize `json_extract_path` comparisons in PostgreSQL\n  * [sql] Optionally log last known call, publish stacktrace in telemetry\n  * [telemetry] Include `:repo` option in telemetry events","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [sql] Ensure `:timeout` option is respected in `Ecto.Adapters.SQL.explain/3`","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.7.2 (2022-01-23) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-7-2-2022-01-23"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [adapters] Support latest `myxql` and `postgrex`","ref":"changelog.html#enhancements"},{"type":"extras","title":"v3.7.1 (2021-10-12) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-7-1-2021-10-12"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [migrations] Add `:cascade` option to `drop`\n  * [migrations] Support `--prefix` in `mix ecto.migrations`\n  * [migrations] Add `--log-migrator-sql` and `--log-migrations-sql`\n  * [mysql] Cache more insert/update queries and allow `:cache_statement` to be set\n  * [mssql] Support more recent tds versions","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [migrations] Consider the database prefix when locking tables","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.7.0 (2021-08-19) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-7-0-2021-08-19"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [mysql] Support lateral joins","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [sql] Fix CTE subqueries not finding parent bindings","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.6.2 (2021-05-28) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-6-2-2021-05-28"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [migration] Improve error message on invalid migration type\n  * [postgres] Avoid duplicate order_by with distinct\n  * [sql] Implement new checked_out? callback required by latest Ecto","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.6.1 (2021-04-12) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-6-1-2021-04-12"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [migrations] Ensure migration_source option is respected in PostgreSQL adapter","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.6.0 (2021-04-03) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-6-0-2021-04-03"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [migrations] Fix a bug where the migration lock would not apply on the first migration (when the schema migrations table is empty). This fix changes how migration tables are locked, therefore let us know of any possible regressions in your workflow","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [migrations] Allow generating migrations from within umbrella app\n  * [postgres] Add `:format` option to PostgreSQL explain\n  * [postgres] Support `:socket_dir` connection option when using `mix ecto.load` or `mix ecto.dump`\n  * [sandbox] Support locally registered processes in `allow/3`\n  * [storage] Do not fail `storage_up` if the user has access to an already-created database\n  * [tds] Support for `:inner_lateral` and `:left_lateral`","ref":"changelog.html#enhancements"},{"type":"extras","title":"v3.5.4 (2020-01-20) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-5-4-2020-01-20"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [mysql] Support defaults for JSON columns\n  * [postgres] Allow Postgrex v1.0","ref":"changelog.html#enhancements"},{"type":"extras","title":"v3.5.3 (2020-10-27) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-5-3-2020-10-27"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [migrations] Pass `:schema_migration` option to repo operations for `prepare_query` checks\n  * [psql] Support `:force_drop` configuration to force a DB to be dropped","ref":"changelog.html#enhancements"},{"type":"extras","title":"v3.5.2 (2020-10-24) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-5-2-2020-10-24"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [migrations] Support `:with` option in `references` for composite foreign keys\n  * [migrations] Support `:match` option in `references`\n  * [tds] Support TDS 3-part and 4-part prefixes","ref":"changelog.html#enhancements"},{"type":"extras","title":"v3.5.1 (2020-10-12) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-5-1-2020-10-12"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [tds] Support explain plan for the TDS adapter","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fix - Changelog for v3.x","doc":"* [migrations] Reload all migrations once the lock is free to avoid running the same migration more than once\n  * [query] Support nested subqueries","ref":"changelog.html#bug-fix"},{"type":"extras","title":"v3.5.0 (2020-10-03) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-5-0-2020-10-03"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [migrations] Add option to skip schema migrations table checks\n  * [migrations] Add `:migration_repo` configuration to allow a different repository to host the schema migrations\n  * [migrations] Support `validate: false` on references and constraints\n  * [migrations] Accept `:migration_primary_key` as false and add `:migration_foreign_key` repo config\n  * [postgres] Support for `:identity` key types in Postgres 10 or later\n  * [postgres] Use IF NOT EXIST when creating index with `create_if_not_exists`, this requires PG 9.5+ or later\n  * [repo] Support `Repo.explain(:all | :update_all | :delete_all, query)` for Ecto adapters\n  * [sandbox] Allow for dynamic repos to be checked out in sandbox","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [migrations] Flush migration commands before executing `before_commit` callback\n  * [migrations] Do not swallow errors when migration lock is disabled","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.4.5 (2020-07-05) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-4-5-2020-07-05"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [ecto] Fix warnings on Elixir v1.11\n  * [migrations] Migration prefix should have higher preference than `default_options`","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.4.4 (2020-05-19) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-4-4-2020-05-19"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [sandbox] Add `Ecto.Adapters.SQL.start_owner!/2` and `Ecto.Adapters.SQL.stop_owner/1`\n  * [myxql] Decode BIT columns when using MyXQL and `:boolean` type\n  * [migrations] Use one line per migration in the schema dump","ref":"changelog.html#enhancements"},{"type":"extras","title":"v3.4.3 (2020-04-27) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-4-3-2020-04-27"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [ecto] Support `as` and `parent_as` from Ecto v3.4.3+\n  * [ecto] Support `x in subquery(query)` from Ecto v3.4.3+","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.4.2 (2020-04-02) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-4-2-2020-04-02"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [myxql] A binary with size should be a varbinary\n  * [mssql] A binary without size should be a varbinary(max)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.4.1 (2020-03-25) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-4-1-2020-03-25"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [myxql] Assume the reference does not change in MyXQL and prepare for v0.4.0","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.4.0 (2020-03-24) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-4-0-2020-03-24"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [adapters] Support Ecto's v3.4 `json_extract_path/2`\n  * [migrations] Support multiple migration paths to be given with `--migration-path`\n  * [mssql] Add built-in support to MSSQL via the TDS adapter\n  * [repo] Support custom options on telemetry","ref":"changelog.html#enhancements"},{"type":"extras","title":"v3.3.4 (2020-02-14) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-3-4-2020-02-14"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [adapters] Support fragments in locks\n  * [migration] Add `:include` option to support covering indexes","ref":"changelog.html#enhancements"},{"type":"extras","title":"v3.3.3 (2020-01-28) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-3-3-2020-01-28"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [myxql] Allow not setting the encoding when creating a database","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [myxql] Removing prefixed table name from constraints on latest MySQL versions\n  * [sql] Fix precedence of `is_nil` when inside a comparison operator","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.3.2 (2019-12-15) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-3-2-2019-12-15"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [adapters] Start StorageSupervisor before using it","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.3.1 (2019-12-15) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-3-1-2019-12-15"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [adapters] Do not leak PIDs on storage commands\n  * [migrations] Use :migration_primary_key in create/1","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.3.0 (2019-12-11) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-3-0-2019-12-11"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [ecto] Upgrade and support Ecto v3.3\n  * [repo] Include `:idle_time` on telemetry measurements\n  * [migration] Support anonymous functions in `Ecto.Migration.execute/2`","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [migration] Ensure that flush() will raise on rollback if called from `change/0`","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.2.2 (2019-11-25) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-2-2-2019-11-25"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [mysql] Support myxql v0.3","ref":"changelog.html#enhancements"},{"type":"extras","title":"v3.2.1 (2019-11-02) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-2-1-2019-11-02"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [migration] Support anonymous functions in execute","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [mix ecto.create] Change default charset in MyXQL to utf8mb4","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.2.0 (2019-09-07) - Changelog for v3.x","doc":"This new version requires Elixir v1.6+. Note also the previously soft-deprecated `Ecto.Adapters.MySQL` has been removed in favor of `Ecto.Adapters.MyXQL`. We announced the intent to remove `Ecto.Adapters.MySQL` back in v3.0 and `Ecto.Adapters.MyXQL` has been tested since then and ready for prime time since v3.1.","ref":"changelog.html#v3-2-0-2019-09-07"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [sql] Use `get_dynamic_repo` on SQL-specific functions\n  * [sql] Respect `Ecto.Type.embed_as/2` choice when loading/dumping embeds (Ecto 3.2+ compat)\n  * [sql] Support CTE expressions (Ecto 3.2+ compat)","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [sql] Fix generated \"COMMENT ON INDEX\" for PostgreSQL","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.1.6 (2019-06-27) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-1-6-2019-06-27"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [sql] Set `cache_statement` for `insert_all`","ref":"changelog.html#enhancements"},{"type":"extras","title":"v3.1.5 (2019-06-13) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-1-5-2019-06-13"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [migration] Add `@disable_migration_lock` to be better handle concurrent indexes\n  * [mysql] Set `cache_statement` for inserts","ref":"changelog.html#enhancements"},{"type":"extras","title":"Deprecations - Changelog for v3.x","doc":"* [mysql] Deprecate Ecto.Adapters.MySQL","ref":"changelog.html#deprecations"},{"type":"extras","title":"v3.1.4 (2019-05-28) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-1-4-2019-05-28"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [migrator] Print warning message if concurrent indexes are used with migration lock","ref":"changelog.html#enhancements"},{"type":"extras","title":"v3.1.3 (2019-05-19) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-1-3-2019-05-19"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [migrator] Add `--migrations-path` to ecto.migrate/ecto.rollback/ecto.migrations Mix tasks","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [migrator] Make sure an unboxed run is performed when running migrations with the ownership pool","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.1.2 (2019-05-11) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-1-2-2019-05-11"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [migrator] Add `Ecto.Migrator.with_repo/2` to start repo and apps\n  * [mix] Add `--skip-if-loaded` for `ecto.load`\n  * [sql] Add `Ecto.Adapters.SQL.table_exists?/2`","ref":"changelog.html#enhancements"},{"type":"extras","title":"v3.1.1 (2019-04-16) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-1-1-2019-04-16"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [repo] Fix backwards incompatible change in Telemetry metadata","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.1.0 (2019-04-02) - Changelog for v3.x","doc":"v3.1 requires Elixir v1.5+.","ref":"changelog.html#v3-1-0-2019-04-02"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [mysql] Introduce Ecto.Adapters.MyXQL as an alternative library for MySQL\n  * [migrations] Run all migrations in subdirectories\n  * [repo] Update to Telemetry v0.4.0 (note the measurements value differ from previous versions)","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [sandbox] Respect `:ownership_timeout` repo configuration on SQL Sandbox\n  * [migrations] Commit and relock after every migration to avoid leaving the DB in an inconsistent state under certain failures","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Backwards incompatible changes - Changelog for v3.x","doc":"* [migrations] If you are creating indexes concurrently, you need to disable the migration lock: `config :app, App.Repo, migration_lock: nil`. This will migrations behave the same way as they did in Ecto 2.0.","ref":"changelog.html#backwards-incompatible-changes"},{"type":"extras","title":"v3.0.5 (2019-02-05) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-0-5-2019-02-05"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [repo] Add `:repo` and `:type` keys to telemetry events\n  * [migrations] Add `:add_if_not_exists` and `:remove_if_exists` to columns in migrations","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [migrations] Load all migrations before running them\n  * [sandbox] Include `:queue_target` and `:queue_interval` in SQL Sandbox checkout","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.0.4 (2018-12-31) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-0-4-2018-12-31"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [repo] Bump telemetry dependency\n  * [migrations] Perform strict argument parsing in `ecto.migrate`, `ecto.rollback`, `ecto.load` and `ecto.dump`","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [migrations] Do not log migration versions query","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Deprecations - Changelog for v3.x","doc":"* [repo] `Telemetry.attach/5` and `Telemetry.attach_many/5` are deprecated in favor of `:telemetry.attach/5` and `:telemetry.attach_many/5`","ref":"changelog.html#deprecations"},{"type":"extras","title":"v3.0.3 (2018-11-29) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-0-3-2018-11-29"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [migration] Support `after_begin` and `before_commit` migration callbacks\n  * [migration] Add `:prefix` option to `references/2`","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [migration] Do not start a transaction for migrated versions if there is no `:migration_lock`\n  * [migration] Fix removing an reference column inside alter table\n  * [migration] Warn on removed `:pool_timeout` option","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.0.2 (2018-11-20) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-0-2-2018-11-20"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [query] Support `Ecto.Query` in `insert_all` values\n  * [migration] Add `Ecto.Migration.repo/0`","ref":"changelog.html#enhancements"},{"type":"extras","title":"v3.0.1 (2018-11-17) - Changelog for v3.x","doc":"","ref":"changelog.html#v3-0-1-2018-11-17"},{"type":"extras","title":"Enhancements - Changelog for v3.x","doc":"* [migrations] Support `drop_if_exists` for constraints","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog for v3.x","doc":"* [migrations] Only commit migration transaction if migration can be inserted into the DB\n  * [migrations] Do not run migrations from `_build` when using Mix\n  * [migrations] Improve errors when checking in already committed sandboxes\n  * [mysql] Do not pass nil for `--user` to mysqldump\n  * [package] Require Ecto 3.0.2 with bug fixes\n  * [package] Require Mariaex 0.9.1 which fixes a bug when used with Ecto 3.0.2\n  * [sandbox] Raise when using sandbox on non-sandbox pools","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v3.0.0 (2018-10-29) - Changelog for v3.x","doc":"* Initial release","ref":"changelog.html#v3-0-0-2018-10-29"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}